from django.db.models.signals import post_delete, post_migrate
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.links import link_acl_list
from mayan.apps.acls.permissions import permission_acl_edit, permission_acl_view
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.classes import (
    MissingItem, ModelField, ModelFieldRelated, ModelProperty, Template
)
from mayan.apps.common.html_widgets import TwoStateWidget
from mayan.apps.common.menus import (
    menu_facet, menu_list_facet, menu_main, menu_object, menu_secondary,
    menu_setup, menu_multi_item, menu_tools
)
from mayan.apps.common.signals import post_initial_setup
from mayan.apps.dashboards.dashboards import dashboard_main
from mayan.apps.converter.links import link_transformation_list
from mayan.apps.converter.permissions import (
    permission_transformation_create,
    permission_transformation_delete, permission_transformation_edit,
    permission_transformation_view,
)
from mayan.apps.events.classes import EventModelRegistry, ModelEventType
from mayan.apps.events.links import (
    link_events_for_object, link_object_event_types_user_subcriptions_list,
)
from mayan.apps.events.permissions import permission_events_view
from mayan.apps.navigation.classes import SourceColumn
from mayan.apps.rest_api.fields import DynamicSerializerField

from .dashboard_widgets import (
    DashboardWidgetDocumentPagesTotal, DashboardWidgetDocumentsInTrash,
    DashboardWidgetDocumentsNewThisMonth,
    DashboardWidgetDocumentsPagesNewThisMonth, DashboardWidgetDocumentsTotal,
    DashboardWidgetDocumentsTypesTotal,
)
from .events import (
    event_document_create, event_document_download,
    event_document_properties_edit, event_document_type_changed,
    event_document_type_created, event_document_type_edited,
    event_document_version_new, event_document_version_revert,
    event_document_view
)
from .handlers import (
    handler_create_default_document_type, handler_create_document_cache,
    handler_remove_empty_duplicates_lists, handler_scan_duplicates_for
)
from .links.document_links import (
    link_document_clear_transformations, link_document_clone_transformations,
    link_document_document_type_edit, link_document_download,
    link_document_edit, link_document_list, link_document_list_recent_access,
    link_document_list_recent_added,
    link_document_multiple_clear_transformations,
    link_document_multiple_document_type_edit,
    link_document_multiple_download, link_document_preview,
    link_document_print, link_document_properties,
    link_document_quick_download
)
from .links.document_version_links import (
    link_document_version_download, link_document_version_list,
    link_document_version_return_document, link_document_version_return_list,
    link_document_version_revert, link_document_version_view
)
from .links.document_type_links import (
    link_document_type_create, link_document_type_delete,
    link_document_type_edit, link_document_type_filename_create,
    link_document_type_filename_delete, link_document_type_filename_edit,
    link_document_type_filename_list, link_document_type_list,
    link_document_type_policies, link_document_type_setup
)
from .links.document_version_page_links import (
    link_document_multiple_update_page_count, link_document_page_disable,
    link_document_page_enable, link_document_page_multiple_enable,
    link_document_page_multiple_disable, link_document_page_navigation_first,
    link_document_page_navigation_last, link_document_page_navigation_next,
    link_document_page_navigation_previous, link_document_page_return,
    link_document_page_rotate_left, link_document_page_rotate_right,
    link_document_page_view, link_document_page_view_reset,
    link_document_page_zoom_in, link_document_page_zoom_out,
    link_document_pages, link_document_update_page_count
)
from .links.duplicated_document_links import (
    link_document_duplicates_list, link_duplicated_document_list,
    link_duplicated_document_scan
)
from .links.favorite_links import (
    link_document_favorites_add, link_document_favorites_remove,
    link_document_list_favorites, link_document_multiple_favorites_add,
    link_document_multiple_favorites_remove
)
from .links.trashed_document_links import (
    link_document_delete, link_document_list_deleted,
    link_document_multiple_delete, link_document_multiple_restore,
    link_document_multiple_trash, link_document_restore, link_document_trash,
    link_trash_can_empty
)

from .menus import menu_documents
from .permissions import (
    permission_document_create, permission_document_delete,
    permission_document_download, permission_document_edit,
    permission_document_new_version, permission_document_print,
    permission_document_properties_edit, permission_document_restore,
    permission_document_tools, permission_document_trash,
    permission_document_type_delete, permission_document_type_edit,
    permission_document_type_view, permission_document_version_revert,
    permission_document_version_view, permission_document_view
)
# Just import to initialize the search models
from .search import document_search, document_page_search  # NOQA
from .signals import post_version_upload
from .statistics import *  # NOQA
from .widgets import (
    DocumentPageThumbnailWidget, widget_document_page_number,
    widget_document_version_page_number
)


def is_document_page_enabled(context):
    return context['object'].enabled


class DocumentsApp(MayanAppConfig):
    app_namespace = 'documents'
    app_url = 'documents'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.documents'
    verbose_name = _('Documents')

    def ready(self):
        super(DocumentsApp, self).ready()

        DeletedDocument = self.get_model(model_name='DeletedDocument')
        Document = self.get_model(model_name='Document')
        DocumentPage = self.get_model(model_name='DocumentPage')
        DocumentPageResult = self.get_model(model_name='DocumentPageResult')
        DocumentType = self.get_model(model_name='DocumentType')
        DocumentTypeFilename = self.get_model(model_name='DocumentTypeFilename')
        DocumentVersion = self.get_model(model_name='DocumentVersion')
        DuplicatedDocument = self.get_model(model_name='DuplicatedDocument')

        DynamicSerializerField.add_serializer(
            klass=Document,
            serializer_class='mayan.apps.documents.serializers.DocumentSerializer'
        )

        EventModelRegistry.register(model=DeletedDocument)
        EventModelRegistry.register(model=Document)
        EventModelRegistry.register(model=DocumentType)
        EventModelRegistry.register(model=DocumentVersion)

        MissingItem(
            label=_('Create a document type'),
            description=_(
                'Every uploaded document must be assigned a document type, '
                'it is the basic way Mayan EDMS categorizes documents.'
            ), condition=lambda: not DocumentType.objects.exists(),
            view='documents:document_type_list'
        )

        ModelEventType.register(
            model=DocumentType, event_types=(
                event_document_create,
                event_document_type_created,
                event_document_type_edited,
            )
        )
        ModelEventType.register(
            model=Document, event_types=(
                event_document_download, event_document_properties_edit,
                event_document_type_changed, event_document_version_new,
                event_document_version_revert, event_document_view
            )
        )

        ModelField(model=Document, name='description')
        ModelField(model=Document, name='date_added')
        ModelField(model=Document, name='deleted_date_time')
        ModelField(
            model=Document, name='document_type'
        )
        ModelField(model=Document, name='in_trash')
        ModelField(model=Document, name='is_stub')
        ModelField(model=Document, name='label')
        ModelField(model=Document, name='language')
        ModelField(model=Document, name='uuid')

        ModelFieldRelated(model=Document, name='document_type__label')
        ModelFieldRelated(
            model=Document,
            name='versions__checksum'
        )
        ModelFieldRelated(
            model=Document, label=_('Versions comment'),
            name='versions__comment'
        )
        ModelFieldRelated(
            model=Document, label=_('Versions encoding'),
            name='versions__encoding'
        )
        ModelFieldRelated(
            model=Document, label=_('Versions mime type'),
            name='versions__mimetype'
        )
        ModelFieldRelated(
            model=Document, label=_('Versions timestamp'),
            name='versions__timestamp'
        )

        ModelProperty(
            description=_('Return the lastest version of the document.'),
            model=Document, label=_('Latest version'), name='latest_version'
        )

        ModelPermission.register(
            model=Document, permissions=(
                permission_acl_edit, permission_acl_view,
                permission_document_delete, permission_document_download,
                permission_document_edit, permission_document_new_version,
                permission_document_print, permission_document_properties_edit,
                permission_document_restore, permission_document_tools,
                permission_document_trash, permission_document_version_revert,
                permission_document_version_view, permission_document_view,
                permission_events_view, permission_transformation_create,
                permission_transformation_delete,
                permission_transformation_edit, permission_transformation_view,
            )
        )

        ModelPermission.register(
            model=DocumentType, permissions=(
                permission_document_create, permission_document_type_delete,
                permission_document_type_edit, permission_document_type_view,
                permission_acl_edit, permission_acl_view,
                permission_events_view,
            )
        )

        ModelPermission.register_inheritance(
            model=Document, related='document_type',
        )
        ModelPermission.register_manager(
            model=Document, manager_name='passthrough'
        )
        ModelPermission.register_inheritance(
            model=DocumentPage, related='document_version__document',
        )
        ModelPermission.register_manager(
            model=DocumentPage, manager_name='passthrough'
        )
        ModelPermission.register_inheritance(
            model=DocumentPageResult, related='document_version__document',
        )
        ModelPermission.register_manager(
            model=DocumentPageResult, manager_name='passthrough'
        )
        ModelPermission.register_inheritance(
            model=DocumentTypeFilename, related='document_type',
        )
        ModelPermission.register_inheritance(
            model=DocumentVersion, related='document',
        )

        # Document and document page thumbnail widget
        document_page_thumbnail_widget = DocumentPageThumbnailWidget()

        # Document
        SourceColumn(
            attribute='label', is_object_absolute_url=True, is_identifier=True,
            is_sortable=True, source=Document
        )
        SourceColumn(
            func=lambda context: document_page_thumbnail_widget.render(
                instance=context['object']
            ), html_extra_classes='text-center document-thumbnail-list',
            label=_('Thumbnail'), source=Document
        )
        SourceColumn(
            attribute='document_type', is_sortable=True, source=Document,
        )
        SourceColumn(
            func=lambda context: widget_document_page_number(
                document=context['object']
            ), label=_('Pages'), source=Document
        )
        SourceColumn(
            attribute='date_added', include_label=True, is_sortable=True,
            source=Document, views=('documents:document_list_recent_added',)
        )
        SourceColumn(
            func=lambda context: DuplicatedDocument.objects.get(
                document=context['object']
            ).documents.count(), include_label=True, label=_('Duplicates'),
            source=Document, views=('documents:duplicated_document_list',)
        )

        # DocumentPage
        SourceColumn(
            attribute='get_label', is_identifier=True,
            is_object_absolute_url=True, source=DocumentPage,
            widget_condition=is_document_page_enabled
        )
        SourceColumn(
            func=lambda context: document_page_thumbnail_widget.render(
                instance=context['object']
            ), html_extra_classes='text-center document-thumbnail-list',
            label=_('Thumbnail'), source=DocumentPage
        )
        SourceColumn(
            attribute='enabled', include_label=True, source=DocumentPage,
            widget=TwoStateWidget
        )
        SourceColumn(
            attribute='page_number', include_label=True, source=DocumentPage
        )

        SourceColumn(
            attribute='get_label', is_identifier=True,
            is_object_absolute_url=True, source=DocumentPageResult
        )
        SourceColumn(
            func=lambda context: document_page_thumbnail_widget.render(
                instance=context['object']
            ), html_extra_classes='text-center document-thumbnail-list',
            label=_('Thumbnail'), source=DocumentPageResult
        )
        SourceColumn(
            attribute='document_version.document.document_type',
            label=_('Type'), source=DocumentPageResult
        )

        # DocumentType
        SourceColumn(
            attribute='label', is_identifier=True, is_sortable=True,
            source=DocumentType
        )

        SourceColumn(
            func=lambda context: context['object'].get_document_count(
                user=context['request'].user
            ), include_label=True, label=_('Documents'), source=DocumentType
        )

        SourceColumn(
            attribute='filename', is_identifier=True, is_sortable=True,
            source=DocumentTypeFilename
        )
        SourceColumn(
            attribute='enabled', include_label=True, is_sortable=True,
            source=DocumentTypeFilename, widget=TwoStateWidget
        )

        # DeletedDocument
        SourceColumn(
            attribute='label', is_identifier=True, is_sortable=True,
            source=DeletedDocument
        )
        SourceColumn(
            attribute='deleted_date_time', include_label=True, order=99,
            source=DeletedDocument
        )

        # DocumentVersion
        SourceColumn(
            source=DocumentVersion, attribute='timestamp', is_identifier=True,
            is_object_absolute_url=True
        )
        SourceColumn(
            func=lambda context: document_page_thumbnail_widget.render(
                instance=context['object']
            ), html_extra_classes='text-center document-thumbnail-list',
            label=_('Thumbnail'), source=DocumentVersion
        )
        SourceColumn(
            func=lambda context: widget_document_version_page_number(
                document_version=context['object']
            ), label=_('Pages'), source=DocumentVersion
        )
        SourceColumn(
            attribute='mimetype', is_sortable=True, source=DocumentVersion
        )
        SourceColumn(
            attribute='encoding', is_sortable=True, source=DocumentVersion
        )
        SourceColumn(
            attribute='comment', is_sortable=True, source=DocumentVersion
        )

        Template(
            name='invalid_document',
            template_name='documents/invalid_document.html'
        )

        dashboard_main.add_widget(
            widget=DashboardWidgetDocumentsTotal, order=0
        )
        dashboard_main.add_widget(
            widget=DashboardWidgetDocumentPagesTotal, order=1
        )
        dashboard_main.add_widget(
            widget=DashboardWidgetDocumentsInTrash, order=2
        )
        dashboard_main.add_widget(
            widget=DashboardWidgetDocumentsTypesTotal, order=3
        )
        dashboard_main.add_widget(
            widget=DashboardWidgetDocumentsNewThisMonth, order=4
        )
        dashboard_main.add_widget(
            widget=DashboardWidgetDocumentsPagesNewThisMonth, order=5
        )

        menu_documents.bind_links(
            links=(
                link_document_list_recent_access,
                link_document_list_recent_added, link_document_list_favorites,
                link_document_list, link_document_list_deleted,
                link_duplicated_document_list,
            )
        )

        menu_main.bind_links(links=(menu_documents,), position=0)

        menu_setup.bind_links(links=(link_document_type_setup,))
        menu_tools.bind_links(
            links=(link_duplicated_document_scan,)
        )

        # Document type links
        menu_list_facet.bind_links(
            links=(
                link_document_type_filename_list,
                link_document_type_policies,
                link_acl_list, link_object_event_types_user_subcriptions_list,
                link_events_for_object,
            ), sources=(DocumentType,)
        )

        menu_object.bind_links(
            links=(
                link_document_type_delete, link_document_type_edit
            ), sources=(DocumentType,)
        )
        menu_object.bind_links(
            links=(
                link_document_type_filename_edit,
                link_document_type_filename_delete
            ), sources=(DocumentTypeFilename,)
        )
        menu_secondary.bind_links(
            links=(link_document_type_list, link_document_type_create),
            sources=(
                DocumentType, 'documents:document_type_create',
                'documents:document_type_list'
            )
        )
        menu_secondary.bind_links(
            links=(link_document_type_filename_create,),
            sources=(
                DocumentTypeFilename, 'documents:document_type_filename_list',
                'documents:document_type_filename_create'
            )
        )
        menu_secondary.bind_links(
            links=(link_trash_can_empty,),
            sources=(
                'documents:document_list_deleted', 'documents:trash_can_empty'
            )
        )

        # Document object links
        menu_object.bind_links(
            links=(
                link_document_favorites_add, link_document_favorites_remove,
                link_document_edit, link_document_document_type_edit,
                link_document_print, link_document_trash,
                link_document_quick_download, link_document_download,
                link_document_clear_transformations,
                link_document_clone_transformations,
                link_document_update_page_count,
            ), sources=(Document,)
        )
        menu_object.bind_links(
            links=(link_document_restore, link_document_delete),
            sources=(DeletedDocument,)
        )

        # Document facet links
        menu_facet.bind_links(
            links=(link_document_duplicates_list, link_acl_list,),
            sources=(Document,)
        )
        menu_facet.bind_links(
            links=(link_document_preview,), sources=(Document,), position=0
        )
        menu_facet.bind_links(
            links=(link_document_properties,), sources=(Document,), position=2
        )
        menu_facet.bind_links(
            links=(
                link_events_for_object,
                link_object_event_types_user_subcriptions_list,
                link_document_version_list,
            ), sources=(Document,), position=2
        )
        menu_facet.bind_links(links=(link_document_pages,), sources=(Document,))

        # Document actions
        menu_object.bind_links(
            links=(
                link_document_version_revert, link_document_version_download
            ),
            sources=(DocumentVersion,)
        )
        menu_multi_item.bind_links(
            links=(
                link_document_multiple_favorites_add,
                link_document_multiple_favorites_remove,
                link_document_multiple_clear_transformations,
                link_document_multiple_trash, link_document_multiple_download,
                link_document_multiple_update_page_count,
                link_document_multiple_document_type_edit,
            ), sources=(Document,)
        )
        menu_multi_item.bind_links(
            links=(
                link_document_multiple_restore, link_document_multiple_delete
            ), sources=(DeletedDocument,)
        )

        # Document pages
        menu_facet.add_unsorted_source(source=DocumentPage)
        menu_facet.bind_links(
            links=(
                link_document_page_rotate_left,
                link_document_page_rotate_right, link_document_page_zoom_in,
                link_document_page_zoom_out, link_document_page_view_reset
            ), sources=('documents:document_page_view',)
        )
        menu_facet.bind_links(
            links=(link_document_page_view,),
            sources=(DocumentPage,)
        )
        menu_facet.bind_links(
            links=(
                link_document_page_navigation_first,
                link_document_page_navigation_previous,
                link_document_page_navigation_next,
                link_document_page_navigation_last
            ), sources=(DocumentPage,)
        )
        menu_list_facet.bind_links(
            links=(link_transformation_list,), sources=(DocumentPage,)
        )
        menu_multi_item.bind_links(
            links=(
                link_document_page_multiple_disable,
                link_document_page_multiple_enable
            ), sources=(DocumentPage,)
        )
        menu_object.bind_links(
            links=(link_document_page_disable, link_document_page_enable),
            sources=(DocumentPage,)
        )
        menu_secondary.bind_links(
            links=(link_document_page_return,),
            sources=(DocumentPage,)
        )

        # Document versions
        menu_list_facet.bind_links(
            links=(link_document_version_view,), sources=(DocumentVersion,)
        )
        menu_secondary.bind_links(
            links=(
                link_document_version_return_document,
                link_document_version_return_list
            ), sources=(DocumentVersion,)
        )

        post_delete.connect(
            dispatch_uid='documents_handler_remove_empty_duplicates_lists',
            receiver=handler_remove_empty_duplicates_lists,
            sender=Document
        )
        post_initial_setup.connect(
            dispatch_uid='documents_handler_create_default_document_type',
            receiver=handler_create_default_document_type
        )
        post_migrate.connect(
            dispatch_uid='documents_handler_create_document_cache',
            receiver=handler_create_document_cache,
        )
        post_version_upload.connect(
            dispatch_uid='documents_handler_scan_duplicates_for',
            receiver=handler_scan_duplicates_for
        )



from django.contrib import messages
from django.contrib.contenttypes.models import ContentType
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from actstream.models import Action, any_stream

from mayan.apps.acls.models import AccessControlList
from mayan.apps.common.generics import (
    FormView, SimpleView, SingleObjectListView
)

from .classes import EventType, ModelEventType
from .forms import (
    EventTypeUserRelationshipFormSet, ObjectEventTypeUserRelationshipFormSet
)
from .icons import (
    icon_events_list, icon_user_notifications_list
)
from .links import link_event_types_subscriptions_list
from .models import StoredEventType
from .permissions import permission_events_view


class EventListView(SingleObjectListView):
    view_permission = permission_events_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'title': _('Events'),
        }

    def get_source_queryset(self):
        return Action.objects.all()


class EventTypeSubscriptionListView(FormView):
    form_class = EventTypeUserRelationshipFormSet
    main_model = 'user'
    submodel = StoredEventType

    def dispatch(self, *args, **kwargs):
        EventType.refresh()
        return super(EventTypeSubscriptionListView, self).dispatch(*args, **kwargs)

    def form_valid(self, form):
        try:
            for instance in form:
                instance.save()
        except Exception as exception:
            messages.error(
                message=_(
                    'Error updating event subscription; %s'
                ) % exception, request=self.request
            )
        else:
            messages.success(
                message=_('Event subscriptions updated successfully'),
                request=self.request
            )

        return super(
            EventTypeSubscriptionListView, self
        ).form_valid(form=form)

    def get_object(self):
        return self.request.user

    def get_extra_context(self):
        return {
            'form_display_mode_table': True,
            'object': self.get_object(),
            'title': _(
                'Event subscriptions'
            ) % self.get_object()
        }

    def get_initial(self):
        obj = self.get_object()
        initial = []

        for element in self.get_queryset():
            initial.append({
                'user': obj,
                'main_model': self.main_model,
                'stored_event_type': element,
            })
        return initial

    def get_queryset(self):
        # Return the queryset by name from the sorted list of the class
        event_type_ids = [event_type.id for event_type in EventType.all()]
        return self.submodel.objects.filter(name__in=event_type_ids)

    def get_post_action_redirect(self):
        return reverse(viewname='user_management:current_user_details')


class NotificationListView(SingleObjectListView):
    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_user_notifications_list,
            'no_results_main_link': link_event_types_subscriptions_list.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'Subscribe to global or object events to receive '
                'notifications.'
            ),
            'no_results_title': _('There are no notifications'),
            'object': self.request.user,
            'title': _('Notifications'),
        }

    def get_source_queryset(self):
        return self.request.user.notifications.all()


class NotificationMarkRead(SimpleView):
    def dispatch(self, *args, **kwargs):
        self.get_queryset().filter(
            pk=self.kwargs['notification_id']
        ).update(read=True)
        return HttpResponseRedirect(
            redirect_to=reverse(viewname='events:user_notifications_list')
        )

    def get_queryset(self):
        return self.request.user.notifications.all()


class NotificationMarkReadAll(SimpleView):
    def dispatch(self, *args, **kwargs):
        self.get_queryset().update(read=True)
        return HttpResponseRedirect(
            redirect_to=reverse(viewname='events:user_notifications_list')
        )

    def get_queryset(self):
        return self.request.user.notifications.all()


class ObjectEventListView(EventListView):
    view_permission = None

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(
            ObjectEventListView, self
        ).dispatch(request, *args, **kwargs)

    def get_extra_context(self):
        context = super(ObjectEventListView, self).get_extra_context()
        context.update(
            {
                'hide_object': True,
                'no_results_icon': icon_events_list,
                'no_results_text': _(
                    'Events are actions that have been performed to this object '
                    'or using this object.'
                ),
                'no_results_title': _('There are no events for this object'),
                'object': self.object,
                'title': _('Events for: %s') % self.object,
            }
        )
        return context

    def get_object(self):
        content_type = get_object_or_404(
            klass=ContentType, app_label=self.kwargs['app_label'],
            model=self.kwargs['model']
        )

        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_events_view,
            queryset=content_type.model_class().objects.all(),
            user=self.request.user
        )
        return get_object_or_404(
            klass=queryset, pk=self.kwargs['object_id']
        )

    def get_source_queryset(self):
        return any_stream(obj=self.object)


class ObjectEventTypeSubscriptionListView(FormView):
    form_class = ObjectEventTypeUserRelationshipFormSet

    def dispatch(self, *args, **kwargs):
        EventType.refresh()
        return super(ObjectEventTypeSubscriptionListView, self).dispatch(
            *args, **kwargs
        )

    def form_valid(self, form):
        try:
            for instance in form:
                instance.save()
        except Exception as exception:
            messages.error(
                message=_(
                    'Error updating object event subscription; %s'
                ) % exception, request=self.request
            )
        else:
            messages.success(
                message=_(
                    'Object event subscriptions updated successfully'
                ), request=self.request
            )

        return super(
            ObjectEventTypeSubscriptionListView, self
        ).form_valid(form=form)

    def get_object(self):
        object_content_type = get_object_or_404(
            klass=ContentType, app_label=self.kwargs['app_label'],
            model=self.kwargs['model']
        )

        try:
            content_object = object_content_type.get_object_for_this_type(
                pk=self.kwargs['object_id']
            )
        except object_content_type.model_class().DoesNotExist:
            raise Http404

        AccessControlList.objects.check_access(
            obj=content_object, permissions=(permission_events_view,),
            user=self.request.user
        )

        return content_object

    def get_extra_context(self):
        return {
            'form_display_mode_table': True,
            'object': self.get_object(),
            'title': _(
                'Event subscriptions for: %s'
            ) % self.get_object()
        }

    def get_initial(self):
        obj = self.get_object()
        initial = []

        for element in self.get_queryset():
            initial.append(
                {
                    'user': self.request.user,
                    'object': obj,
                    'stored_event_type': element,
                }
            )
        return initial

    def get_queryset(self):
        return ModelEventType.get_for_instance(instance=self.get_object())


class CurrentUserEventListView(ObjectEventListView):
    def get_object(self):
        return self.request.user


class VerbEventListView(SingleObjectListView):
    def get_extra_context(self):
        return {
            'hide_object': True,
            'title': _(
                'Events of type: %s'
            ) % EventType.get(name=self.kwargs['verb']),
        }

    def get_source_queryset(self):
        return Action.objects.filter(verb=self.kwargs['verb'])



import logging

from django.utils.termcolors import colorize

PALETTE = {
    'CRITICAL': {'fg': 'red', 'opts': ('bold', 'blink', 'reverse')},
    'DEBUG': {'fg': 'cyan'},
    'ERROR': {'fg': 'red', 'opts': ('bold',)},
    'INFO': {'fg': 'white'},
    'SUCCESS': {'fg': 'green'},
    'WARNING': {'fg': 'yellow', 'opts': ('bold', 'underscore')},
}


class ColorFormatter(logging.Formatter):
    def format(self, record):
        record.msg = colorize(
            text=record.msg, **PALETTE.get(record.levelname, {})
        )
        return super(ColorFormatter, self).format(record)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from ..icons import (
    icon_document_page_navigation_first,
    icon_document_page_navigation_last, icon_document_page_navigation_next,
    icon_document_page_navigation_previous, icon_document_page_return,
    icon_document_page_rotate_left, icon_document_page_rotate_right,
    icon_document_page_zoom_in, icon_document_page_zoom_out,
)
from ..permissions import (
    permission_document_edit, permission_document_tools,
    permission_document_view
)
from ..settings import setting_zoom_max_level, setting_zoom_min_level


def is_first_page(context):
    return context['resolved_object'].siblings.first() == context['resolved_object']


def is_last_page(context):
    return context['resolved_object'].siblings.last() == context['resolved_object']


def is_max_zoom(context):
    return context['zoom'] >= setting_zoom_max_level.value


def is_min_zoom(context):
    return context['zoom'] <= setting_zoom_min_level.value


def is_document_page_enabled(context):
    return context['resolved_object'].enabled


def is_document_page_disabled(context):
    return not context['resolved_object'].enabled


link_document_page_disable = Link(
    condition=is_document_page_enabled,
    icon_class_path='mayan.apps.documents.icons.icon_document_page_disable',
    kwargs={'document_page_id': 'resolved_object.id'},
    permissions=(permission_document_edit,), text=_('Disable page'),
    view='documents:document_page_disable'
)
link_document_page_multiple_disable = Link(
    icon_class_path='mayan.apps.documents.icons.icon_document_page_disable',
    text=_('Disable pages'),
    view='documents:document_page_multiple_disable'
)
link_document_page_enable = Link(
    condition=is_document_page_disabled,
    icon_class_path='mayan.apps.documents.icons.icon_document_page_enable',
    kwargs={'document_page_id': 'resolved_object.id'},
    permissions=(permission_document_edit,), text=_('Enable page'),
    view='documents:document_page_enable'
)
link_document_page_multiple_enable = Link(
    icon_class_path='mayan.apps.documents.icons.icon_document_page_enable',
    text=_('Enable pages'),
    view='documents:document_page_multiple_enable'
)
link_document_page_navigation_first = Link(
    args='resolved_object.pk', conditional_disable=is_first_page,
    icon_class=icon_document_page_navigation_first,
    keep_query=True, permissions=(permission_document_view,),
    text=_('First page'), view='documents:document_page_navigation_first',
)
link_document_page_navigation_last = Link(
    args='resolved_object.pk', conditional_disable=is_last_page,
    icon_class=icon_document_page_navigation_last,
    keep_query=True, text=_('Last page'),
    permissions=(permission_document_view,),
    view='documents:document_page_navigation_last',
)
link_document_page_navigation_previous = Link(
    args='resolved_object.pk', conditional_disable=is_first_page,
    icon_class=icon_document_page_navigation_previous,
    keep_query=True, permissions=(permission_document_view,),
    text=_('Previous page'),
    view='documents:document_page_navigation_previous',
)
link_document_page_navigation_next = Link(
    args='resolved_object.pk', conditional_disable=is_last_page,
    icon_class=icon_document_page_navigation_next,
    keep_query=True, text=_('Next page'),
    permissions=(permission_document_view,),
    view='documents:document_page_navigation_next',
)
link_document_page_return = Link(
    args='resolved_object.document.pk', icon_class=icon_document_page_return,
    permissions=(permission_document_view,), text=_('Document'),
    view='documents:document_preview',
)
link_document_page_rotate_left = Link(
    args='resolved_object.pk', icon_class=icon_document_page_rotate_left,
    keep_query=True, permissions=(permission_document_view,),
    text=_('Rotate left'), view='documents:document_page_rotate_left',
)
link_document_page_rotate_right = Link(
    args='resolved_object.pk', icon_class=icon_document_page_rotate_right,
    keep_query=True, permissions=(permission_document_view,),
    text=_('Rotate right'), view='documents:document_page_rotate_right',
)
link_document_page_view = Link(
    conditional_disable=is_document_page_disabled,
    icon_class_path='mayan.apps.documents.icons.icon_document_page_view',
    permissions=(permission_document_view,), text=_('Page image'),
    view='documents:document_page_view', args='resolved_object.pk'
)
link_document_page_view_reset = Link(
    icon_class_path='mayan.apps.documents.icons.icon_document_page_view_reset',
    permissions=(permission_document_view,), text=_('Reset view'),
    view='documents:document_page_view_reset', args='resolved_object.pk'
)
link_document_page_zoom_in = Link(
    args='resolved_object.pk', conditional_disable=is_max_zoom,
    icon_class=icon_document_page_zoom_in, keep_query=True,
    permissions=(permission_document_view,), text=_('Zoom in'),
    view='documents:document_page_zoom_in',
)
link_document_page_zoom_out = Link(
    args='resolved_object.pk', conditional_disable=is_min_zoom,
    icon_class=icon_document_page_zoom_out, keep_query=True,
    permissions=(permission_document_view,), text=_('Zoom out'),
    view='documents:document_page_zoom_out',
)
link_document_pages = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.documents.icons.icon_document_pages',
    permissions=(permission_document_view,), text=_('Pages'),
    view='documents:document_pages',
)
link_document_multiple_update_page_count = Link(
    icon_class_path='mayan.apps.documents.icons.icon_document_page_count_update',
    text=_('Recalculate page count'),
    view='documents:document_multiple_update_page_count'
)
link_document_update_page_count = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.documents.icons.icon_document_page_count_update',
    permissions=(permission_document_tools,),
    text=_('Recalculate page count'),
    view='documents:document_update_page_count'
)



from django.utils.translation import ugettext_lazy as _

DEFAULT_COMMON_HOME_VIEW = 'common:home'
DEFAULT_FIREFOX_GECKODRIVER_PATH = '/usr/bin/geckodriver'
DELETE_STALE_UPLOADS_INTERVAL = 60 * 10  # 10 minutes
DJANGO_SQLITE_BACKEND = 'django.db.backends.sqlite3'

MSG_MIME_TYPES = (
    'application/vnd.ms-outlook', 'application/vnd.ms-office'
)

LIST_MODE_CHOICE_LIST = 'list'
LIST_MODE_CHOICE_ITEM = 'item'

MESSAGE_DEPRECATION_WARNING = _(
    'This feature has been deprecated and will be removed in a future version.'
)
MESSAGE_SQLITE_WARNING = _(
    'Your database backend is set to use SQLite. SQLite should only be used '
    'for development and testing, not for production.'
)

PK_LIST_SEPARATOR = ','

STORAGE_NAME_COMMON_SHARED_UPLOADED_FILE = 'common__shareduploadedfile'

TEXT_LIST_AS_ITEMS_PARAMETER = '_list_mode'
TEXT_LIST_AS_ITEMS_VARIABLE_NAME = 'list_as_items'
TEXT_CHOICE_ITEMS = 'items'
TEXT_CHOICE_LIST = 'list'

TEXT_SORT_FIELD_PARAMETER = '_sort_field'
TEXT_SORT_FIELD_VARIABLE_NAME = 'sort_field'
TEXT_SORT_ORDER_CHOICE_ASCENDING = 'asc'
TEXT_SORT_ORDER_CHOICE_DESCENDING = 'desc'
TEXT_SORT_ORDER_PARAMETER = '_sort_order'
TEXT_SORT_ORDER_VARIABLE_NAME = 'sort_order'

TIME_DELTA_UNIT_DAYS = 'days'
TIME_DELTA_UNIT_HOURS = 'hours'
TIME_DELTA_UNIT_MINUTES = 'minutes'

TIME_DELTA_UNIT_CHOICES = (
    (TIME_DELTA_UNIT_DAYS, _('Days')),
    (TIME_DELTA_UNIT_HOURS, _('Hours')),
    (TIME_DELTA_UNIT_MINUTES, _('Minutes')),
)
UPLOAD_EXPIRATION_INTERVAL = 60 * 60 * 24 * 7  # 7 days



import yaml

from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.serialization import yaml_load

from .models import LayerTransformation
from .transformations import BaseTransformation


class LayerTransformationSelectForm(forms.Form):
    def __init__(self, *args, **kwargs):
        layer = kwargs.pop('layer')
        super(LayerTransformationSelectForm, self).__init__(*args, **kwargs)
        self.fields[
            'transformation'
        ].choices = BaseTransformation.get_transformation_choices(layer=layer)

    transformation = forms.ChoiceField(
        choices=(), help_text=_('Available transformations for this layer.'),
        label=_('Transformation'),
    )


class LayerTransformationForm(forms.ModelForm):
    class Meta:
        fields = ('arguments', 'order')
        model = LayerTransformation

    def __init__(self, *args, **kwargs):
        transformation_name = kwargs.pop('transformation_name', None)
        super(LayerTransformationForm, self).__init__(*args, **kwargs)

        if not transformation_name:
            # Get the template name when the transformation is being edited.
            template_name = getattr(
                self.instance.get_transformation_class(), 'template_name',
                None
            )
        else:
            # Get the template name when the transformation is being created
            template_name = getattr(
                BaseTransformation.get(name=transformation_name),
                'template_name', None
            )

        if template_name:
            self.fields['arguments'].widget.attrs['class'] = 'hidden'
            self.fields['order'].widget.attrs['class'] = 'hidden'

    def clean(self):
        try:
            yaml_load(stream=self.cleaned_data['arguments'])
        except yaml.YAMLError:
            raise ValidationError(
                _(
                    '"%s" not a valid entry.'
                ) % self.cleaned_data['arguments']
            )



from django.db import models, migrations


def operation_create_ocr_setting_for_existing_document_types(apps, schema_editor):
    DocumentType = apps.get_model(
        app_label='documents', model_name='DocumentType'
    )
    DocumentTypeSettings = apps.get_model(
        app_label='ocr', model_name='DocumentTypeSettings'
    )

    for document_type in DocumentType.objects.using(schema_editor.connection.alias).all():
        try:
            DocumentTypeSettings.objects.using(
                schema_editor.connection.alias
            ).create(document_type=document_type)
        except DocumentTypeSettings.DoesNotExist:
            pass


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0016_auto_20150708_0325'),
        ('ocr', '0003_auto_20150617_0401'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentTypeSettings',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'auto_ocr', models.BooleanField(
                        default=True,
                        verbose_name='Automatically queue newly created '
                        'documents for OCR.'
                    )
                ),
                (
                    'document_type', models.OneToOneField(
                        on_delete=models.CASCADE,
                        related_name='ocr_settings',
                        to='documents.DocumentType',
                        verbose_name='Document type'
                    )
                ),
            ],
            options={
                'verbose_name': 'Document type settings',
                'verbose_name_plural': 'Document types settings',
            },
            bases=(models.Model,),
        ),
        migrations.RunPython(
            code=operation_create_ocr_setting_for_existing_document_types
        ),
    ]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

namespace = Namespace(label=_('Mirroring'), name='mirroring')

setting_document_lookup_cache_timeout = namespace.add_setting(
    global_name='MIRRORING_DOCUMENT_CACHE_LOOKUP_TIMEOUT', default=10,
    help_text=_('Time in seconds to cache the path lookup to a document.'),
)
setting_node_lookup_cache_timeout = namespace.add_setting(
    global_name='MIRRORING_NODE_CACHE_LOOKUP_TIMEOUT', default=10,
    help_text=_('Time in seconds to cache the path lookup to an index node.'),
)



from importlib import import_module
import json
import logging

from django.apps import apps
from django.utils import six
from django.utils.encoding import force_text
from django.utils.text import format_lazy
from django.utils.translation import ugettext_lazy as _

from .exceptions import QuotaExceeded
from .handlers import handler_process_quota_signal

logger = logging.getLogger(name=__name__)


__ALL__ = ('QuotaBackend',)


class QuotaBackendMetaclass(type):
    _registry = {}

    def __new__(mcs, name, bases, attrs):
        new_class = super(QuotaBackendMetaclass, mcs).__new__(
            mcs, name, bases, attrs
        )
        if not new_class.__module__ == 'mayan.apps.quotas.classes':
            mcs._registry[
                '{}.{}'.format(new_class.__module__, name)
            ] = new_class
            new_class.id = '{}.{}'.format(new_class.__module__, name)

        return new_class


class QuotaBackendBase(object):
    """
    Base class for the mailing backends. This class is mainly a wrapper
    for other Django backends that adds a few metadata to specify the
    fields it needs to be instanciated at runtime.

    The fields attribute is a list of dictionaries with the format:
    {
        'name': ''  # Field internal name
        'label': ''  # Label to show to users
        'class': ''  # Field class to use. Field classes are Python dot
                       paths to Django's form fields.
        'initial': ''  # Field initial value
        'default': ''  # Default value.
    }

    """
    fields = {}
    signal = None
    widgets = {}


class QuotaBackend(
    six.with_metaclass(QuotaBackendMetaclass, QuotaBackendBase)
):
    @staticmethod
    def _queryset_to_text_list(queryset):
        return ','.join(list(map(str, queryset))) or _('none')

    @staticmethod
    def initialize():
        for app in apps.get_app_configs():
            try:
                import_module('{}.quota_backends'.format(app.name))
            except ImportError as exception:
                if force_text(exception) not in ('No module named quota_backends', 'No module named \'{}.quota_backends\''.format(app.name)):
                    logger.error(
                        'Error importing %s quota_backends.py file; %s',
                        app.name, exception
                    )

        for backend in QuotaBackend.get_all():
            backend._initialize()

        QuotaBackend.connect_signals()

    @staticmethod
    def connect_signals():
        for backend in QuotaBackend.get_all():
            if backend.signal:
                backend.signal.connect(
                    dispatch_uid='quotas_handler_process_signal',
                    receiver=handler_process_quota_signal,
                    sender=backend.sender
                )

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    @classmethod
    def get_all(cls):
        return sorted(
            cls._registry.values(), key=lambda x: x.label
        )

    @classmethod
    def get_fields(cls):
        return cls.fields

    @classmethod
    def get_field_order(cls):
        return cls.field_order

    @classmethod
    def get_widgets(cls):
        return cls.widgets

    @classmethod
    def as_choices(cls):
        return [
            (
                backend.id, backend.label
            ) for backend in QuotaBackend.get_all()
        ]

    @classmethod
    def _initialize(cls):
        """
        Allow the quota backend to run code when the app initializes.
        """

    @classmethod
    def create(cls, **kwargs):
        Quota = apps.get_model(app_label='quotas', model_name='Quota')
        return Quota.objects.create(
            backend_path=cls.get_dotted_path(),
            backend_data=json.dumps(obj=kwargs)
        )

    @classmethod
    def get_dotted_path(cls):
        return '{}.{}'.format(cls.__module__, cls.__name__)

    @classmethod
    def get_instances(cls):
        Quota = apps.get_model(app_label='quotas', model_name='Quota')
        return Quota.objects.filter(backend_path=cls.get_dotted_path())

    def _allowed_filter_display(self):
        return ''

    def filters(self):
        result = self.get_filter_text()

        return format_lazy(
            ', '.join(['{}'] * len(result)), *result.values()
        )

    def get_filter_text(self):
        return {'limit': self._allowed_filter_display()}

    def process(self, **kwargs):
        if self._usage() >= self._allowed():
            raise QuotaExceeded(self.error_message)

    def usage(self):
        return _('Does not apply')


class NullBackend(QuotaBackend):
    label = _('Null backend')
    signal = None

    def __init__(self, *args, **kwargs):
        super(NullBackend, self).__init__()

    def display(self):
        return _('Null backend')



from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.search import document_search, document_page_search

__all__ = ()

document_search.add_model_field(
    field='workflows__log_entries__comment', label=_(
        'Workflow transition comment'
    )
)
document_page_search.add_model_field(
    field='document_version__document__workflows__log_entries__comment',
    label=_('Workflow transition comment')
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from ..permissions import permission_document_view


link_document_list_favorites = Link(
    icon_class_path='mayan.apps.documents.icons.icon_favorite_document_list',
    text=_('Favorites'),
    view='documents:document_list_favorites'
)
link_document_favorites_add = Link(
    args='resolved_object.id',
    icon_class_path='mayan.apps.documents.icons.icon_favorite_document_add',
    permissions=(permission_document_view,), text=_('Add to favorites'),
    view='documents:document_add_to_favorites',
)
link_document_favorites_remove = Link(
    args='resolved_object.id',
    icon_class_path='mayan.apps.documents.icons.icon_favorite_document_remove',
    permissions=(permission_document_view,), text=_('Remove from favorites'),
    view='documents:document_remove_from_favorites',
)
link_document_multiple_favorites_add = Link(
    text=_('Add to favorites'),
    icon_class_path='mayan.apps.documents.icons.icon_favorite_document_add',
    view='documents:document_multiple_add_to_favorites',
)
link_document_multiple_favorites_remove = Link(
    text=_('Remove from favorites'),
    icon_class_path='mayan.apps.documents.icons.icon_favorite_document_remove',
    view='documents:document_multiple_remove_from_favorites',
)



from .classes import PythonDependency

PythonDependency(
    module=__name__, name='node-semver', version_string='==0.8.0'
)
PythonDependency(
    module=__name__, name='packaging', version_string='==20.3'
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Menu

from .icons import icon_menu_documents

menu_documents = Menu(
    icon_class=icon_menu_documents, label=_('Documents'),
    name='documents'
)



from django.apps import apps

from ..finders import MayanAppDirectoriesFinder

from .base import BaseTestCase


class MayanAppDirectoriesFinderTestCase(BaseTestCase):
    def test_app_detection(self):
        app_list = []

        for app_config in apps.get_app_configs():
            if getattr(app_config, 'has_static_media', False):
                if app_config.name not in app_list:
                    app_list.append(app_config.name)

        test_finder = MayanAppDirectoriesFinder()

        for finder_app in test_finder.apps:
            if finder_app.startswith('mayan'):
                self.assertTrue(
                    finder_app in app_list, msg='"{}" is missing'.format(finder_app)
                )

        self.assertEqual(
            len(test_finder.apps), len(test_finder.storages)
        )



from django.utils.six import StringIO

from django.core import management

from mayan.apps.common.tests.base import BaseTestCase

from ..classes import PlatformTemplate

TEST_TEMPLATE_LABEL = 'test template label'
TEST_TEMPLATE_NAME = 'test_template_name'
TEST_TEMPLATE_STRING = '''
test template string
test template variable: {{ test_template_variable }}
'''
TEST_TEMPLATE_VARIABLE_VALUE = 'test_variable_value'
TEST_TEMPLATE_VARIABLE_VALUE_ALT = 'test_variable_value_alt'
TEST_TEMPLATE_STRING_RENDER = '''
test template string
test template variable: {}
'''.format(TEST_TEMPLATE_VARIABLE_VALUE)
TEST_TEMPLATE_STRING_RENDER_ALT = '''
test template string
test template variable: {}
'''.format(TEST_TEMPLATE_VARIABLE_VALUE_ALT)


class TestPlatformTemplate(PlatformTemplate):
    context = {'test_template_variable': TEST_TEMPLATE_VARIABLE_VALUE}
    label = TEST_TEMPLATE_LABEL
    name = TEST_TEMPLATE_NAME
    template_string = TEST_TEMPLATE_STRING


PlatformTemplate.register(klass=TestPlatformTemplate)


class PlatformTemplateManagementCommandTestCase(BaseTestCase):
    def test_platform_template_simple(self):
        output = StringIO()
        args = (TEST_TEMPLATE_NAME,)
        options = {
            'stdout': output
        }
        management.call_command('platformtemplate', *args, **options)
        self.assertEqual(output.getvalue(), TEST_TEMPLATE_STRING_RENDER)

    def test_platform_template_context(self):
        output = StringIO()
        args = (
            TEST_TEMPLATE_NAME, '--context',
            'test_template_variable: {}'.format(
                TEST_TEMPLATE_VARIABLE_VALUE_ALT
            )
        )
        options = {
            'stdout': output
        }
        management.call_command('platformtemplate', *args, **options)
        self.assertEqual(output.getvalue(), TEST_TEMPLATE_STRING_RENDER_ALT)



from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .icons import (
    icon_events_list, icon_events_for_object,
    icon_event_types_subscriptions_list,
    icon_object_event_types_user_subcriptions_list,
    icon_user_notifications_list
)
from .permissions import permission_events_view


def get_kwargs_factory(variable_name):
    def get_kwargs(context):
        ContentType = apps.get_model(
            app_label='contenttypes', model_name='ContentType'
        )

        content_type = ContentType.objects.get_for_model(
            model=context[variable_name]
        )
        return {
            'app_label': '"{}"'.format(content_type.app_label),
            'model': '"{}"'.format(content_type.model),
            'object_id': '{}.pk'.format(variable_name)
        }

    return get_kwargs


def get_unread_notification_count(context):
    Notification = apps.get_model(
        app_label='events', model_name='Notification'
    )
    return Notification.objects.filter(
        user=context.request.user
    ).filter(read=False).count()


link_current_user_events = Link(
    icon_class=icon_events_list, text=_('My events'),
    view='events:current_user_events'
)
link_events_details = Link(
    text=_('Events'), view='events:events_list'
)
link_events_for_object = Link(
    icon_class=icon_events_for_object,
    kwargs=get_kwargs_factory('resolved_object'),
    permissions=(permission_events_view,), text=_('Events'),
    view='events:events_for_object',
)
link_events_list = Link(
    icon_class=icon_events_list, permissions=(permission_events_view,),
    text=_('Events'), view='events:events_list'
)
link_event_types_subscriptions_list = Link(
    icon_class=icon_event_types_subscriptions_list,
    text=_('Event subscriptions'),
    view='events:event_types_user_subcriptions_list'
)
link_notification_mark_read = Link(
    args='object.pk', text=_('Mark as seen'),
    view='events:notification_mark_read'
)
link_notification_mark_read_all = Link(
    text=_('Mark all as seen'), view='events:notification_mark_read_all'
)
link_object_event_types_user_subcriptions_list = Link(
    icon_class=icon_object_event_types_user_subcriptions_list,
    kwargs=get_kwargs_factory('resolved_object'),
    permissions=(permission_events_view,), text=_('Subscriptions'),
    view='events:object_event_types_user_subcriptions_list',
)
link_user_notifications_list = Link(
    badge_text=get_unread_notification_count,
    icon_class=icon_user_notifications_list, text='',
    view='events:user_notifications_list'
)



from django.test import override_settings

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.documents.tests.literals import TEST_DEU_DOCUMENT_PATH

from .literals import (
    TEST_DOCUMENT_CONTENT, TEST_DOCUMENT_CONTENT_DEU_1,
    TEST_DOCUMENT_CONTENT_DEU_2
)


@override_settings(OCR_AUTO_OCR=True)
class DocumentOCRTestCase(DocumentTestMixin, BaseTestCase):
    # PyOCR's leak descriptor in get_available_languages and image_to_string
    # Disable descriptor leak test until fixed in upstream
    _skip_file_descriptor_test = True

    def test_ocr_language_backends_end(self):
        content = self.test_document.pages.first().ocr_content.content
        self.assertTrue(TEST_DOCUMENT_CONTENT in content)


@override_settings(OCR_AUTO_OCR=True)
class GermanOCRSupportTestCase(DocumentTestMixin, BaseTestCase):
    # PyOCR's leak descriptor in get_available_languages and image_to_string
    # Disable descriptor leak test until fixed in upstream
    _skip_file_descriptor_test = True

    auto_upload_test_document = False

    def setUp(self):
        super(GermanOCRSupportTestCase, self).setUp()

        with open(TEST_DEU_DOCUMENT_PATH, mode='rb') as file_object:
            self.test_document = self.test_document_type.new_document(
                file_object=file_object, language='deu'
            )

    def test_ocr_language_backends_end(self):
        content = self.test_document.pages.first().ocr_content.content

        self.assertTrue(
            TEST_DOCUMENT_CONTENT_DEU_1 in content
        )
        self.assertTrue(
            TEST_DOCUMENT_CONTENT_DEU_2 in content
        )



from django.template.loader import render_to_string

from .permissions import permission_cabinet_view


def widget_document_cabinets(document, user):
    """
    A tag widget that displays the tags for the given document
    """
    return render_to_string(
        template_name='cabinets/document_cabinets_widget.html', context={
            'cabinets': document.get_cabinets(
                permission=permission_cabinet_view, user=user
            )
        }
    )


def widget_single_cabinet(cabinet):
    return render_to_string(
        template_name='cabinets/cabinet_widget.html', context={
            'cabinet': cabinet
        }
    )



from mayan.apps.common.tests.base import BaseTestCase

from ..classes import ModelPermission


class ModelPermissionTestCase(BaseTestCase):
    def test_model_permission_get_classes_as_content_type(self):
        self.assertNotEqual(
            ModelPermission.get_classes(as_content_type=True).count(), 0
        )



from mayan.apps.common.tests.base import GenericViewTestCase

from ..classes import Worker, CeleryQueue
from ..permissions import permission_task_view

from .literals import TEST_QUEUE_LABEL, TEST_QUEUE_NAME, TEST_WORKER_NAME


class TaskManagerTestMixin(object):
    def _create_test_queue(self):
        self.test_worker = Worker(name=TEST_WORKER_NAME)
        self.test_queue = CeleryQueue(
            label=TEST_QUEUE_LABEL, name=TEST_QUEUE_NAME,
            worker=self.test_worker
        )


class TaskManagerViewTestCase(TaskManagerTestMixin, GenericViewTestCase):
    def setUp(self):
        super(TaskManagerViewTestCase, self).setUp()
        self._create_test_queue()

    def _request_queue_list(self):
        return self.get(
            viewname='task_manager:queue_list', follow=True
        )

    def test_queue_list_view_no_permissions(self):
        response = self._request_queue_list()

        self.assertEqual(response.status_code, 403)

    def test_queue_list_view_with_permissions(self):
        self.grant_permission(permission=permission_task_view)

        response = self._request_queue_list()
        self.assertContains(
            response, text=self.test_queue.name, status_code=200
        )



from .base import *  # NOQA

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.oracle',
        'NAME': 'xe',
        'USER': 'system',
        'HOST': '127.0.0.1',
        'PORT': '49161',
        'PASSWORD': 'oracle',
    }
}



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

namespace = Namespace(
    label=_('REST API'), name='rest_api', version='0001'
)

setting_disable_links = namespace.add_setting(
    global_name='REST_API_DISABLE_LINKS', default=False,
    help_text=_(
        'Disable the REST API links in the tools menu.'
    ),
)



from django.conf.urls import url

from .api_views import (
    APIClassPermissionList, APIObjectACLListView,
    APIObjectACLPermissionListView, APIObjectACLPermissionView,
    APIObjectACLView
)
from .views import (
    ACLCreateView, ACLDeleteView, ACLListView, ACLPermissionsView
)

urlpatterns = [
    url(
        regex=r'^acls/(?P<acl_id>\d+)/delete/$', name='acl_delete',
        view=ACLDeleteView.as_view()
    ),
    url(
        regex=r'^acls/(?P<acl_id>\d+)/permissions/$', name='acl_permissions',
        view=ACLPermissionsView.as_view()
    ),
    url(
        regex=r'^apps/(?P<app_label>[-\w]+)/models/(?P<model_name>[-\w]+)/objects/(?P<object_id>\d+)/acls/$',
        name='acl_list', view=ACLListView.as_view()
    ),
    url(
        regex=r'^apps/(?P<app_label>[-\w]+)/models/(?P<model_name>[-\w]+)/objects/(?P<object_id>\d+)/acls/create/$',
        name='acl_create', view=ACLCreateView.as_view()
    )
]

api_urls = [
    url(
        regex=r'^objects/(?P<app_label>[-\w]+)/(?P<model_name>[-\w]+)/permissions/$',
        name='class-permission-list', view=APIClassPermissionList.as_view()
    ),
    url(
        regex=r'^objects/(?P<app_label>[-\w]+)/(?P<model_name>[-\w]+)/(?P<object_id>\d+)/acls/$',
        name='accesscontrollist-list', view=APIObjectACLListView.as_view()
    ),
    url(
        regex=r'^objects/(?P<app_label>[-\w]+)/(?P<model_name>[-\w]+)/(?P<object_id>\d+)/acls/(?P<pk>\d+)/$',
        name='accesscontrollist-detail', view=APIObjectACLView.as_view()
    ),
    url(
        regex=r'^objects/(?P<app_label>[-\w]+)/(?P<model_name>[-\w]+)/(?P<object_id>\d+)/acls/(?P<pk>\d+)/permissions/$',
        name='accesscontrollist-permission-list',
        view=APIObjectACLPermissionListView.as_view()
    ),
    url(
        regex=r'^objects/(?P<app_label>[-\w]+)/(?P<model_name>[-\w]+)/(?P<object_id>\d+)/acls/(?P<pk>\d+)/permissions/(?P<permission_pk>\d+)/$',
        name='accesscontrollist-permission-detail',
        view=APIObjectACLPermissionView.as_view()
    ),
]



from mayan.apps.common.tests.base import BaseTestCase

from .literals import (
    TEST_BOOTSTAP_SETTING_NAME, TEST_SETTING_VALUE,
    TEST_SETTING_VALUE_OVERRIDE
)
from .mixins import BoostrapSettingTestMixin, SmartSettingTestMixin


class BoostrapSettingTestCase(
    BoostrapSettingTestMixin, SmartSettingTestMixin, BaseTestCase
):
    def setUp(self):
        super(BoostrapSettingTestCase, self).setUp()
        self._register_test_boostrap_setting()
        self._create_test_bootstrap_singleton()

    def test_bootstrap_environment_overrides_config(self):
        self._set_environment_variable(
            name='MAYAN_{}'.format(TEST_BOOTSTAP_SETTING_NAME),
            value=TEST_SETTING_VALUE_OVERRIDE
        )

        self.test_setting_global_name = TEST_BOOTSTAP_SETTING_NAME
        self.test_config_value = TEST_SETTING_VALUE
        self._create_test_config_file(
            callback=self.test_setting_namespace_singleton.update_globals
        )

        self.assertEqual(
            self.test_globals[TEST_BOOTSTAP_SETTING_NAME],
            TEST_SETTING_VALUE_OVERRIDE
        )

    def test_bootstrap_config_overrides_settings(self):
        self.test_globals[TEST_BOOTSTAP_SETTING_NAME] = TEST_SETTING_VALUE

        self.test_setting_global_name = TEST_BOOTSTAP_SETTING_NAME
        self.test_config_value = TEST_SETTING_VALUE_OVERRIDE
        self._create_test_config_file(
            callback=self.test_setting_namespace_singleton.update_globals
        )

        self.assertEqual(
            self.test_globals[TEST_BOOTSTAP_SETTING_NAME],
            TEST_SETTING_VALUE_OVERRIDE
        )

    def test_bootstrap_settings_overrides_default(self):
        self.test_globals[
            TEST_BOOTSTAP_SETTING_NAME
        ] = TEST_SETTING_VALUE_OVERRIDE

        self.test_setting_namespace_singleton.update_globals()

        self.assertEqual(
            self.test_globals[TEST_BOOTSTAP_SETTING_NAME],
            TEST_SETTING_VALUE_OVERRIDE
        )

    def test_bootstrap_default(self):
        self.test_setting_namespace_singleton.update_globals()

        self.assertEqual(
            self.test_globals[TEST_BOOTSTAP_SETTING_NAME], 'value default'
        )



import logging

from django.contrib import messages
from django.shortcuts import reverse
from django.template import RequestContext
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.acls.models import AccessControlList
from mayan.apps.common.generics import (
    MultipleObjectFormActionView, MultipleObjectConfirmActionView,
    SingleObjectCreateView, SingleObjectEditView, SingleObjectListView
)
from mayan.apps.common.mixins import ExternalObjectMixin
from mayan.apps.documents.models import Document
from mayan.apps.documents.views.document_views import DocumentListView

from .forms import TagMultipleSelectionForm
from .icons import icon_menu_tags, icon_document_tag_remove_submit
from .links import link_document_tag_multiple_attach, link_tag_create
from .models import Tag
from .permissions import (
    permission_tag_attach, permission_tag_create, permission_tag_delete,
    permission_tag_edit, permission_tag_remove, permission_tag_view
)

logger = logging.getLogger(name=__name__)


class TagAttachActionView(MultipleObjectFormActionView):
    form_class = TagMultipleSelectionForm
    model = Document
    object_permission = permission_tag_attach
    pk_url_kwarg = 'document_id'
    success_message = _('Tag attach request performed on %(count)d document')
    success_message_plural = _(
        'Tag attach request performed on %(count)d documents'
    )

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'submit_label': _('Attach'),
            'title': ungettext(
                singular='Attach tags to %(count)d document',
                plural='Attach tags to %(count)d documents',
                number=queryset.count()
            ) % {
                'count': queryset.count(),
            }
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _('Attach tags to document: %s') % queryset.first()
                }
            )

        return result

    def get_form_extra_kwargs(self):
        queryset = self.object_list
        result = {
            'help_text': _('Tags to be attached.'),
            'permission': permission_tag_attach,
            'queryset': Tag.objects.all(),
            'user': self.request.user
        }

        if queryset.count() == 1:
            result.update(
                {
                    'queryset': Tag.objects.exclude(
                        pk__in=queryset.first().tags.all()
                    )
                }
            )

        return result

    def get_post_action_redirect(self):
        queryset = self.object_list
        if queryset.count() == 1:
            return reverse(
                viewname='tags:document_tag_list', kwargs={
                    'document_id': queryset.first().pk
                }
            )
        else:
            return super(TagAttachActionView, self).get_post_action_redirect()

    def object_action(self, form, instance):
        attached_tags = instance.get_tags(
            permission=permission_tag_attach, user=self.request.user
        )

        for tag in form.cleaned_data['tags']:
            AccessControlList.objects.check_access(
                obj=tag, permissions=(permission_tag_attach,),
                user=self.request.user
            )

            if tag in attached_tags:
                messages.warning(
                    message=_(
                        'Document "%(document)s" is already tagged as '
                        '"%(tag)s"'
                    ) % {
                        'document': instance, 'tag': tag
                    }, request=self.request
                )
            else:
                tag.attach_to(document=instance, user=self.request.user)
                messages.success(
                    message=_(
                        'Tag "%(tag)s" attached successfully to document '
                        '"%(document)s".'
                    ) % {
                        'document': instance, 'tag': tag
                    }, request=self.request
                )


class TagCreateView(SingleObjectCreateView):
    extra_context = {'title': _('Create tag')}
    fields = ('label', 'color')
    model = Tag
    post_action_redirect = reverse_lazy(viewname='tags:tag_list')
    view_permission = permission_tag_create

    def get_save_extra_data(self):
        return {'_user': self.request.user}


class TagDeleteActionView(MultipleObjectConfirmActionView):
    model = Tag
    object_permission = permission_tag_delete
    pk_url_kwarg = 'tag_id'
    post_action_redirect = reverse_lazy(viewname='tags:tag_list')
    success_message = _('Tag delete request performed on %(count)d tag')
    success_message_plural = _(
        'Tag delete request performed on %(count)d tags'
    )

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'delete_view': True,
            'message': _('Will be removed from all documents.'),
            'title': ungettext(
                singular='Delete the selected tag?',
                plural='Delete the selected tags?',
                number=queryset.count()
            )
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _('Delete tag: %s') % queryset.first()
                }
            )

        return result

    def object_action(self, instance, form=None):
        try:
            instance.delete()
            messages.success(
                message=_(
                    'Tag "%s" deleted successfully.'
                ) % instance, request=self.request
            )
        except Exception as exception:
            messages.error(
                message=_('Error deleting tag "%(tag)s": %(error)s') % {
                    'tag': instance, 'error': exception
                }, request=self.request
            )


class TagEditView(SingleObjectEditView):
    fields = ('label', 'color')
    model = Tag
    object_permission = permission_tag_edit
    pk_url_kwarg = 'tag_id'
    post_action_redirect = reverse_lazy(viewname='tags:tag_list')

    def get_extra_context(self):
        return {
            'object': self.object,
            'title': _('Edit tag: %s') % self.object,
        }

    def get_save_extra_data(self):
        return {'_user': self.request.user}


class TagListView(SingleObjectListView):
    object_permission = permission_tag_view

    def get_extra_context(self):
        return {
            'hide_link': True,
            'hide_object': True,
            'no_results_icon': icon_menu_tags,
            'no_results_main_link': link_tag_create.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'Tags are color coded properties that can be attached or '
                'removed from documents.'
            ),
            'no_results_title': _('No tags available'),
            'title': _('Tags'),
        }

    def get_source_queryset(self):
        return self.get_tag_queryset()

    def get_tag_queryset(self):
        return Tag.objects.all()


class TagDocumentListView(ExternalObjectMixin, DocumentListView):
    external_object_class = Tag
    external_object_permission = permission_tag_view
    external_object_pk_url_kwarg = 'tag_id'

    def get_document_queryset(self):
        return self.get_tag().get_documents(user=self.request.user).all()

    def get_extra_context(self):
        context = super(TagDocumentListView, self).get_extra_context()
        context.update(
            {
                'object': self.get_tag(),
                'title': _('Documents with the tag: %s') % self.get_tag(),
            }
        )
        return context

    def get_tag(self):
        return self.external_object


class DocumentTagListView(ExternalObjectMixin, TagListView):
    external_object_class = Document
    external_object_permission = permission_tag_view
    external_object_pk_url_kwarg = 'document_id'

    def get_extra_context(self):
        context = super(DocumentTagListView, self).get_extra_context()
        context.update(
            {
                'hide_link': True,
                'no_results_main_link': link_document_tag_multiple_attach.resolve(
                    context=RequestContext(
                        self.request, {'object': self.external_object}
                    )
                ),
                'no_results_title': _('Document has no tags attached'),
                'object': self.external_object,
                'title': _(
                    'Tags for document: %s'
                ) % self.external_object,
            }
        )
        return context

    def get_source_queryset(self):
        return self.external_object.get_tags(
            permission=permission_tag_view, user=self.request.user
        )


class TagRemoveActionView(MultipleObjectFormActionView):
    form_class = TagMultipleSelectionForm
    model = Document
    object_permission = permission_tag_remove
    pk_url_kwarg = 'document_id'
    success_message = _('Tag remove request performed on %(count)d document')
    success_message_plural = _(
        'Tag remove request performed on %(count)d documents'
    )

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'submit_icon_class': icon_document_tag_remove_submit,
            'submit_label': _('Remove'),
            'title': ungettext(
                singular='Remove tags from %(count)d document',
                plural='Remove tags from %(count)d documents',
                number=queryset.count()
            ) % {
                'count': queryset.count(),
            }
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _('Remove tags from document: %s') % queryset.first()
                }
            )

        return result

    def get_form_extra_kwargs(self):
        queryset = self.object_list
        result = {
            'help_text': _('Tags to be removed.'),
            'permission': permission_tag_remove,
            'queryset': Tag.objects.all(),
            'user': self.request.user
        }

        if queryset.count() == 1:
            result.update(
                {
                    'queryset': queryset.first().tags.all()
                }
            )

        return result

    def get_post_action_redirect(self):
        queryset = self.object_list
        if queryset.count() == 1:
            return reverse(
                viewname='tags:document_tag_list', kwargs={
                    'document_id': queryset.first().pk
                }
            )
        else:
            return super(TagRemoveActionView, self).get_post_action_redirect()

    def object_action(self, form, instance):
        attached_tags = instance.get_tags(
            permission=permission_tag_remove, user=self.request.user
        )

        for tag in form.cleaned_data['tags']:
            AccessControlList.objects.check_access(
                obj=tag, permissions=(permission_tag_remove,),
                user=self.request.user
            )

            if tag not in attached_tags:
                messages.warning(
                    message=_(
                        'Document "%(document)s" wasn\'t tagged as "%(tag)s'
                    ) % {
                        'document': instance, 'tag': tag
                    }, request=self.request
                )
            else:
                tag.remove_from(document=instance, user=self.request.user)
                messages.success(
                    message=_(
                        'Tag "%(tag)s" removed successfully from document '
                        '"%(document)s".'
                    ) % {
                        'document': instance, 'tag': tag
                    }, request=self.request
                )



from django.template import Library

from ..literals import (
    IMPERSONATE_VARIABLE_ID, IMPERSONATE_VARIABLE_PERMANENT
)

register = Library()


@register.simple_tag(takes_context=True)
def authentication_impersonation_check(context):
    request = getattr(context, 'request', None)

    if request:
        user_id = request.session.get(IMPERSONATE_VARIABLE_ID)
        impersonate_permanent_session = IMPERSONATE_VARIABLE_PERMANENT in request.session

        if user_id and not impersonate_permanent_session:
            return context.request.user
        else:
            return False
    else:
        return False



from django.contrib import admin

from .models import Comment


@admin.register(Comment)
class CommentAdmin(admin.ModelAdmin):
    date_hierarchy = 'submit_date'
    list_display = ('document', 'submit_date', 'user', 'comment')
    list_filter = ('user',)
    readonly_fields = ('document', 'submit_date', 'user', 'comment')
    search_fields = ('comment',)



from django.utils.translation import ugettext_lazy as _

from .classes import LayerLink
from .layers import layer_saved_transformations


def conditional_active(context, resolved_link):
    return resolved_link.link.view == resolved_link.current_view_name and context.get('layer_name', None) == resolved_link.link.layer_name


link_transformation_delete = LayerLink(
    action='delete', kwargs={
        'layer_name': 'layer_name', 'transformation_id': 'resolved_object.pk'
    },
    icon_class_path='mayan.apps.converter.icons.icon_transformation_delete',
    layer=layer_saved_transformations,
    tags='dangerous', text=_('Delete'), view='converter:transformation_delete'
)
link_transformation_edit = LayerLink(
    action='edit', kwargs={
        'layer_name': 'layer_name', 'transformation_id': 'resolved_object.pk'
    }, icon_class_path='mayan.apps.converter.icons.icon_transformation_edit',
    layer=layer_saved_transformations,
    text=_('Edit'), view='converter:transformation_edit'
)
link_transformation_list = LayerLink(
    action='list', conditional_active=conditional_active,
    layer=layer_saved_transformations, text=_('Transformations'),
    view='converter:transformation_list'
)
link_transformation_select = LayerLink(
    action='select',
    icon_class_path='mayan.apps.converter.icons.icon_transformation_select',
    layer=layer_saved_transformations, text=_('Select new transformation'),
    view='converter:transformation_select'
)



from django.utils.module_loading import import_string

from .settings import (
    setting_search_backend, setting_search_backend_arguments
)

search_backend = import_string(dotted_path=setting_search_backend.value)(
    **setting_search_backend_arguments.value
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

from .setting_migrations import OCRSettingMigration

namespace = Namespace(
    label=_('OCR'), migration_class=OCRSettingMigration, name='ocr',
    version='0002'
)

setting_ocr_backend = namespace.add_setting(
    global_name='OCR_BACKEND',
    default='mayan.apps.ocr.backends.tesseract.Tesseract',
    help_text=_('Full path to the backend to be used to do OCR.')
)
setting_ocr_backend_arguments = namespace.add_setting(
    global_name='OCR_BACKEND_ARGUMENTS',
    default={}
)
setting_auto_ocr = namespace.add_setting(
    global_name='OCR_AUTO_OCR', default=True,
    help_text=_(
        'Set new document types to perform OCR automatically by default.'
    )
)



from django.template import Library

from ..models import Message

register = Library()


@register.inclusion_tag('motd/messages.html')
def motd():
    return {'messages': Message.objects.get_for_now()}



from django.template.loader import render_to_string

from mayan.apps.documents.settings import (
    setting_preview_width, setting_preview_height, setting_thumbnail_width,
    setting_thumbnail_height
)


class StagingFileThumbnailWidget(object):
    def render(self, instance):
        return render_to_string(
            template_name='documents/widgets/document_thumbnail.html',
            context={
                'container_class': 'staging-file-thumbnail-container',
                'disable_title_link': True,
                'gallery_name': 'sources:staging_list',
                'instance': instance,
                'size_preview_width': setting_preview_width.value,
                'size_preview_height': setting_preview_height.value,
                'size_thumbnail_width': setting_thumbnail_width.value,
                'size_thumbnail_height': setting_thumbnail_height.value,
            }
        )



from django.apps import apps
from django.conf import settings
from django.core import management
from django.utils import timezone, translation


def handler_pre_initial_setup(sender, **kwargs):
    management.call_command(command_name='migrate', interactive=False)


def handler_pre_upgrade(sender, **kwargs):
    management.call_command(
        command_name='migrate', fake_initial=True, interactive=False
    )
    management.call_command(command_name='purgeperiodictasks')


def handler_user_locale_profile_session_config(sender, request, user, **kwargs):
    UserLocaleProfile = apps.get_model(
        app_label='common', model_name='UserLocaleProfile'
    )

    user_locale_profile, created = UserLocaleProfile.objects.get_or_create(
        user=user
    )

    if not created and user_locale_profile.timezone and user_locale_profile.language:
        # Don't try to set locale preferences for new users or existing users
        # that have not set any preferences
        timezone.activate(user_locale_profile.timezone)
        translation.activate(user_locale_profile.language)

        if hasattr(request, 'session'):
            request.session[
                translation.LANGUAGE_SESSION_KEY
            ] = user_locale_profile.language
            request.session[
                settings.TIMEZONE_SESSION_KEY
            ] = user_locale_profile.timezone
        else:
            request.set_cookie(
                settings.LANGUAGE_COOKIE_NAME, user_locale_profile.language
            )
            request.set_cookie(
                settings.TIMEZONE_COOKIE_NAME, user_locale_profile.timezone
            )


def handler_user_locale_profile_create(sender, instance, created, **kwargs):
    UserLocaleProfile = apps.get_model(
        app_label='common', model_name='UserLocaleProfile'
    )

    if created:
        UserLocaleProfile.objects.create(user=instance)



from django.test import override_settings

from mayan.apps.converter.layers import layer_saved_transformations
from mayan.apps.converter.permissions import permission_transformation_delete
from mayan.apps.converter.tests.mixins import LayerTestMixin

from ..models import DeletedDocument, Document, DocumentType
from ..permissions import (
    permission_document_create, permission_document_download,
    permission_document_print, permission_document_properties_edit,
    permission_document_tools, permission_document_view,
    permission_empty_trash
)

from .base import GenericDocumentViewTestCase
from .literals import (
    TEST_DOCUMENT_TYPE_2_LABEL, TEST_SMALL_DOCUMENT_FILENAME
)
from .mixins import DocumentViewTestMixin


class DocumentViewTestCase(
    LayerTestMixin, DocumentViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_properties_view_no_permissions(self):
        response = self._request_test_document_properties_view()
        self.assertEqual(response.status_code, 404)

    def test_document_properties_view_with_permissions(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_properties_view()
        self.assertContains(
            response=response, status_code=200, text=self.test_document.label
        )

    @override_settings(DOCUMENTS_LANGUAGE='fra')
    def test_document_properties_view_setting_non_us_language_with_permissions(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_properties_view()
        self.assertContains(
            response=response, status_code=200, text=self.test_document.label
        )
        self.assertContains(
            response=response, status_code=200,
            text='Language:</label>\n                \n                \n                    English'
        )

    @override_settings(DOCUMENTS_LANGUAGE='fra')
    def test_document_properties_edit_get_view_setting_non_us_language_with_permissions(self):
        self.grant_access(
            permission=permission_document_properties_edit,
            obj=self.test_document_type
        )
        response = self._request_test_document_properties_edit_get_view()
        self.assertContains(
            response=response, status_code=200,
            text='<option value="eng" selected>English</option>',
        )

    def test_document_list_view_no_permissions(self):
        response = self._request_test_document_list_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(response.context['object_list'].count(), 0)

    def test_document_list_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_list_view()
        self.assertContains(
            response=response, status_code=200, text=self.test_document.label
        )

    def test_document_document_type_change_post_view_no_permissions(self):
        self.assertEqual(
            self.test_document.document_type, self.test_document_type
        )

        document_type_2 = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE_2_LABEL
        )

        response = self._request_test_document_type_edit_post_view(
            document_type=document_type_2
        )
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            Document.objects.get(pk=self.test_document.pk).document_type,
            self.test_document_type
        )

    def test_document_document_type_change_post_view_with_permissions(self):
        self.assertEqual(
            self.test_document.document_type, self.test_document_type
        )

        document_type_2 = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE_2_LABEL
        )

        self.grant_access(
            obj=self.test_document, permission=permission_document_properties_edit
        )
        self.grant_access(
            obj=document_type_2, permission=permission_document_create
        )

        response = self._request_test_document_type_edit_post_view(
            document_type=document_type_2
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            Document.objects.get(pk=self.test_document.pk).document_type,
            document_type_2
        )

    def test_document_document_type_change_view_get_no_permissions(self):
        response = self._request_test_document_type_edit_get_view(
        )
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            Document.objects.get(pk=self.test_document.pk).document_type,
            self.test_document_type
        )

    def test_document_document_type_change_view_get_with_permissions(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_properties_edit
        )
        response = self._request_test_document_type_edit_get_view(
        )
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            Document.objects.get(pk=self.test_document.pk).document_type,
            self.test_document_type
        )

    def test_document_multiple_document_type_change_view_no_permission(self):
        self.assertEqual(
            Document.objects.first().document_type, self.test_document_type
        )

        document_type_2 = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE_2_LABEL
        )

        response = self._request_test_document_multiple_type_edit(
            document_type=document_type_2
        )
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            Document.objects.first().document_type, self.test_document_type
        )

    def test_document_multiple_document_type_change_view_with_permission(self):
        self.assertEqual(
            Document.objects.first().document_type, self.test_document_type
        )

        document_type_2 = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE_2_LABEL
        )

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_properties_edit
        )
        self.grant_access(
            obj=document_type_2, permission=permission_document_create
        )

        response = self._request_test_document_multiple_type_edit(
            document_type=document_type_2
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            Document.objects.first().document_type, document_type_2
        )

    def test_document_download_form_get_view_no_permission(self):
        response = self._request_test_document_download_form_get_view()
        self.assertEqual(response.status_code, 404)

    def test_document_download_form_get_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_download
        )

        response = self._request_test_document_download_form_get_view()
        self.assertContains(
            response=response, status_code=200, text=self.test_document.label
        )

    def test_document_download_form_post_view_no_permission(self):
        response = self._request_test_document_download_form_post_view()
        self.assertEqual(response.status_code, 404)

    def test_document_download_form_post_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_download
        )
        response = self._request_test_document_download_form_post_view()
        self.assertEqual(response.status_code, 302)

    def test_document_download_view_no_permission(self):
        response = self._request_test_document_download_view()
        self.assertEqual(response.status_code, 404)

    def test_document_download_view_with_permission(self):
        # Set the expected_content_types for
        # common.tests.mixins.ContentTypeCheckMixin
        self.expected_content_types = (
            self.test_document.file_mimetype,
        )

        self.grant_access(
            obj=self.test_document, permission=permission_document_download
        )

        response = self._request_test_document_download_view()
        self.assertEqual(response.status_code, 200)

        with self.test_document.open() as file_object:
            self.assert_download_response(
                response=response, content=file_object.read(),
                filename=TEST_SMALL_DOCUMENT_FILENAME,
                mime_type=self.test_document.file_mimetype
            )

    def test_document_multiple_download_view_no_permission(self):
        response = self._request_test_document_multiple_download_view()
        self.assertEqual(response.status_code, 404)

    def test_document_multiple_download_view_with_permission(self):
        # Set the expected_content_types for
        # common.tests.mixins.ContentTypeCheckMixin
        self.expected_content_types = (
            self.test_document.file_mimetype,
        )
        self.grant_access(
            obj=self.test_document, permission=permission_document_download
        )

        response = self._request_test_document_multiple_download_view()
        self.assertEqual(response.status_code, 200)

        with self.test_document.open() as file_object:
            self.assert_download_response(
                response=response, content=file_object.read(),
                filename=TEST_SMALL_DOCUMENT_FILENAME,
                mime_type=self.test_document.file_mimetype
            )

    def test_document_update_page_count_view_no_permission(self):
        self.test_document.pages.all().delete()
        self.assertEqual(self.test_document.pages.count(), 0)

        response = self._request_test_document_update_page_count_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(self.test_document.pages.count(), 0)

    def test_document_update_page_count_view_with_permission(self):
        page_count = self.test_document.pages.count()
        self.test_document.pages.all().delete()
        self.assertEqual(self.test_document.pages.count(), 0)

        self.grant_access(
            obj=self.test_document, permission=permission_document_tools
        )

        response = self._request_test_document_update_page_count_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(self.test_document.pages.count(), page_count)

    def test_document_multiple_update_page_count_view_no_permission(self):
        self.test_document.pages.all().delete()
        self.assertEqual(self.test_document.pages.count(), 0)

        response = self._request_test_document_multiple_update_page_count_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(self.test_document.pages.count(), 0)

    def test_document_multiple_update_page_count_view_with_permission(self):
        page_count = self.test_document.pages.count()
        self.test_document.pages.all().delete()
        self.assertEqual(self.test_document.pages.count(), 0)

        self.grant_access(
            obj=self.test_document, permission=permission_document_tools
        )

        response = self._request_test_document_multiple_update_page_count_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(self.test_document.pages.count(), page_count)

    def test_document_clear_transformations_view_no_permission(self):
        self._create_document_transformation()

        transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self.test_document.pages.first()
        ).count()

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_clear_transformations_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            transformation_count,
            layer_saved_transformations.get_transformations_for(
                obj=self.test_document.pages.first()
            ).count()
        )

    def test_document_clear_transformations_view_with_access(self):
        self._create_document_transformation()

        transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self.test_document.pages.first()
        ).count()

        self.grant_access(
            obj=self.test_document,
            permission=permission_transformation_delete
        )
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_clear_transformations_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            transformation_count - 1,
            layer_saved_transformations.get_transformations_for(
                obj=self.test_document.pages.first()
            ).count()
        )

    def test_document_multiple_clear_transformations_view_no_permission(self):
        self._create_document_transformation()

        transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self.test_document.pages.first()
        ).count()

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_multiple_clear_transformations()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            transformation_count,
            layer_saved_transformations.get_transformations_for(
                obj=self.test_document.pages.first()
            ).count()
        )

    def test_document_multiple_clear_transformations_view_with_access(self):
        self._create_document_transformation()

        transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self.test_document.pages.first()
        ).count()

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        self.grant_access(
            obj=self.test_document, permission=permission_transformation_delete
        )

        response = self._request_test_document_multiple_clear_transformations()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            transformation_count - 1,
            layer_saved_transformations.get_transformations_for(
                obj=self.test_document.pages.first()
            ).count()
        )

    def test_trash_can_empty_view_no_permission(self):
        self.test_document.delete()
        self.assertEqual(DeletedDocument.objects.count(), 1)

        response = self._request_empty_trash_view()
        self.assertEqual(response.status_code, 403)
        self.assertEqual(DeletedDocument.objects.count(), 1)

    def test_trash_can_empty_view_with_permission(self):
        self.test_document.delete()
        self.assertEqual(DeletedDocument.objects.count(), 1)

        self.grant_permission(permission=permission_empty_trash)

        response = self._request_empty_trash_view()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(DeletedDocument.objects.count(), 0)
        self.assertEqual(Document.objects.count(), 0)

    def test_document_print_view_no_access(self):
        response = self._request_test_document_print_view()
        self.assertEqual(response.status_code, 403)

    def test_document_print_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_print
        )

        response = self._request_test_document_print_view()
        self.assertEqual(response.status_code, 200)

    def test_document_preview_view_no_permission(self):
        response = self._request_test_document_preview_view()
        self.assertEqual(response.status_code, 404)

    def test_document_preview_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_preview_view()
        self.assertContains(
            response=response, status_code=200, text=self.test_document.label
        )



from django.conf.urls import url

from .api_views import (
    APITrashedDocumentListView, APIDeletedDocumentRestoreView,
    APIDeletedDocumentView, APIDocumentDocumentTypeChangeView,
    APIDocumentDownloadView, APIDocumentView, APIDocumentListView,
    APIDocumentVersionDownloadView, APIDocumentPageImageView,
    APIDocumentPageView, APIDocumentTypeDocumentListView,
    APIDocumentTypeListView, APIDocumentTypeView,
    APIDocumentVersionsListView, APIDocumentVersionPageListView,
    APIDocumentVersionView, APIRecentDocumentListView
)
from .views.document_page_views import (
    DocumentPageDisable, DocumentPageEnable, DocumentPageListView,
    DocumentPageNavigationFirst, DocumentPageNavigationLast,
    DocumentPageNavigationNext, DocumentPageNavigationPrevious,
    DocumentPageRotateLeftView, DocumentPageRotateRightView,
    DocumentPageView, DocumentPageViewResetView, DocumentPageZoomInView,
    DocumentPageZoomOutView
)
from .views.document_type_views import (
    DocumentTypeCreateView, DocumentTypeDeleteView,
    DocumentTypeDeletionPoliciesEditView, DocumentTypeDocumentListView,
    DocumentTypeEditView, DocumentTypeFilenameCreateView,
    DocumentTypeFilenameDeleteView, DocumentTypeFilenameEditView,
    DocumentTypeFilenameListView, DocumentTypeListView
)
from .views.document_version_views import (
    DocumentVersionDownloadFormView, DocumentVersionDownloadView,
    DocumentVersionListView, DocumentVersionRevertView, DocumentVersionView,
)
from .views.document_views import (
    DocumentDocumentTypeEditView, DocumentDownloadFormView,
    DocumentDownloadView, DocumentListView, DocumentPreviewView,
    DocumentPrint, DocumentPropertiesEditView,
    DocumentTransformationsClearView, DocumentTransformationsCloneView,
    DocumentUpdatePageCountView, DocumentView, RecentAccessDocumentListView,
    RecentAddedDocumentListView
)
from .views.duplicated_document_views import (
    DocumentDuplicatesListView, DuplicatedDocumentListView,
    ScanDuplicatedDocuments
)
from .views.favorite_document_views import (
    FavoriteAddView, FavoriteDocumentListView, FavoriteRemoveView
)
from .views.trashed_document_views import (
    DocumentTrashView, EmptyTrashCanView, TrashedDocumentDeleteView,
    TrashedDocumentListView, TrashedDocumentRestoreView
)

urlpatterns_document_types = [
    url(
        regex=r'^document_types/$', name='document_type_list',
        view=DocumentTypeListView.as_view()
    ),
    url(
        regex=r'^document_types/create/$', name='document_type_create',
        view=DocumentTypeCreateView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/delete/$',
        name='document_type_delete', view=DocumentTypeDeleteView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/deletion_policies/$',
        name='document_type_policies',
        view=DocumentTypeDeletionPoliciesEditView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/documents/$',
        name='document_type_document_list',
        view=DocumentTypeDocumentListView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/edit/$',
        name='document_type_edit', view=DocumentTypeEditView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/filenames/$',
        name='document_type_filename_list',
        view=DocumentTypeFilenameListView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/filenames/create/$',
        name='document_type_filename_create',
        view=DocumentTypeFilenameCreateView.as_view()
    ),
    url(
        regex=r'^document_types/filenames/(?P<document_type_filename_id>\d+)/delete/$',
        name='document_type_filename_delete',
        view=DocumentTypeFilenameDeleteView.as_view()
    ),
    url(
        regex=r'^document_types/filenames/(?P<document_type_filename_id>\d+)/edit/$',
        name='document_type_filename_edit',
        view=DocumentTypeFilenameEditView.as_view()
    ),
]

urlpatterns_documents = [
    url(
        regex=r'^documents/$', name='document_list',
        view=DocumentListView.as_view()
    ),
    url(
        regex=r'^documents/recent_access/$',
        name='document_list_recent_access',
        view=RecentAccessDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/recent_added/$',
        name='document_list_recent_added',
        view=RecentAddedDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/preview/$',
        name='document_preview', view=DocumentPreviewView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/properties/$',
        name='document_properties', view=DocumentView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/properties/edit/$',
        name='document_edit', view=DocumentPropertiesEditView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/type/$',
        name='document_document_type_edit',
        view=DocumentDocumentTypeEditView.as_view()
    ),
    url(
        regex=r'^documents/multiple/type/$',
        name='document_multiple_document_type_edit',
        view=DocumentDocumentTypeEditView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/print/$',
        name='document_print', view=DocumentPrint.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/reset_page_count/$',
        name='document_update_page_count',
        view=DocumentUpdatePageCountView.as_view()
    ),
    url(
        regex=r'^documents/multiple/reset_page_count/$',
        name='document_multiple_update_page_count',
        view=DocumentUpdatePageCountView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/download/form/$',
        name='document_download_form',
        view=DocumentDownloadFormView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/download/$',
        name='document_download', view=DocumentDownloadView.as_view()
    ),
    url(
        regex=r'^documents/multiple/download/form/$',
        name='document_multiple_download_form',
        view=DocumentDownloadFormView.as_view()
    ),
    url(
        regex=r'^documents/multiple/download/$',
        name='document_multiple_download',
        view=DocumentDownloadView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/transformations/clear/$',
        name='document_clear_transformations',
        view=DocumentTransformationsClearView.as_view()
    ),
    url(
        regex=r'^documents/multiple/clear_transformations/$',
        name='document_multiple_clear_transformations',
        view=DocumentTransformationsClearView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/transformations/clone/$',
        name='document_clone_transformations',
        view=DocumentTransformationsCloneView.as_view()
    ),
]

urlpatterns_document_pages = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/pages/$',
        name='document_pages', view=DocumentPageListView.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/$',
        name='document_page_view', view=DocumentPageView.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/disable/$',
        name='document_page_disable', view=DocumentPageDisable.as_view()
    ),
    url(
        regex=r'^documents/pages/multiple/disable/$',
        name='document_page_multiple_disable',
        view=DocumentPageDisable.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/enable/$',
        name='document_page_enable', view=DocumentPageEnable.as_view()
    ),
    url(
        regex=r'^documents/pages/multiple/enable/$',
        name='document_page_multiple_enable',
        view=DocumentPageEnable.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/navigation/next/$',
        name='document_page_navigation_next',
        view=DocumentPageNavigationNext.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/navigation/previous/$',
        name='document_page_navigation_previous',
        view=DocumentPageNavigationPrevious.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/navigation/first/$',
        name='document_page_navigation_first',
        view=DocumentPageNavigationFirst.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/navigation/last/$',
        name='document_page_navigation_last',
        view=DocumentPageNavigationLast.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/zoom/in/$',
        name='document_page_zoom_in', view=DocumentPageZoomInView.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/zoom/out/$',
        name='document_page_zoom_out', view=DocumentPageZoomOutView.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/rotate/left/$',
        name='document_page_rotate_left',
        view=DocumentPageRotateLeftView.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/rotate/right/$',
        name='document_page_rotate_right',
        view=DocumentPageRotateRightView.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/reset/$',
        name='document_page_view_reset',
        view=DocumentPageViewResetView.as_view()
    ),
]

urlpatterns_document_versions = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/versions/$',
        name='document_version_list', view=DocumentVersionListView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/download/form/$',
        name='document_version_download_form',
        view=DocumentVersionDownloadFormView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/$',
        name='document_version_view', view=DocumentVersionView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/download/$',
        name='document_version_download',
        view=DocumentVersionDownloadView.as_view()
    ),
    url(
        regex=r'^documents/versions/multiple/download/$',
        name='document_multiple_version_download',
        view=DocumentVersionDownloadView.as_view()
    ),
    url(
        regex=r'^documents/versions/(?P<document_version_id>\d+)/revert/$',
        name='document_version_revert',
        view=DocumentVersionRevertView.as_view()
    ),
]

urlpatterns_duplicated_documents = [
    url(
        regex=r'^documents/duplicated/$',
        name='duplicated_document_list',
        view=DuplicatedDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/duplicates/$',
        name='document_duplicates_list',
        view=DocumentDuplicatesListView.as_view()
    ),
    url(
        regex=r'^tools/documents/duplicated/scan/$',
        name='duplicated_document_scan',
        view=ScanDuplicatedDocuments.as_view()
    ),
]

urlpatterns_favorite_documents = [
    url(
        regex=r'^documents/favorites/$', name='document_list_favorites',
        view=FavoriteDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/add_to_favorites/$',
        name='document_add_to_favorites', view=FavoriteAddView.as_view()
    ),
    url(
        regex=r'^documents/multiple/add_to_favorites/$',
        name='document_multiple_add_to_favorites',
        view=FavoriteAddView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/remove_from_favorites/$',
        name='document_remove_from_favorites',
        view=FavoriteRemoveView.as_view()
    ),
    url(
        regex=r'^documents/multiple/remove_from_favorites/$',
        name='document_multiple_remove_from_favorites',
        view=FavoriteRemoveView.as_view()
    ),
]

urlpatterns_trashed_documents = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/trash/$',
        name='document_trash', view=DocumentTrashView.as_view()
    ),
    url(
        regex=r'^documents/multiple/trash/$', name='document_multiple_trash',
        view=DocumentTrashView.as_view()
    ),
    url(
        regex=r'^trashed_documents/$', name='document_list_deleted',
        view=TrashedDocumentListView.as_view()
    ),
    url(
        regex=r'^trashed_documents/(?P<document_id>\d+)/restore/$',
        name='document_restore', view=TrashedDocumentRestoreView.as_view()
    ),
    url(
        regex=r'^trashed_documents/multiple/restore/$',
        name='document_multiple_restore',
        view=TrashedDocumentRestoreView.as_view()
    ),
    url(
        regex=r'^trashed_documents/(?P<document_id>\d+)/delete/$',
        name='document_delete', view=TrashedDocumentDeleteView.as_view()
    ),
    url(
        regex=r'^trashed_documents/multiple/delete/$',
        name='document_multiple_delete',
        view=TrashedDocumentDeleteView.as_view()
    ),
    url(
        regex=r'^trash_can/empty/$', name='trash_can_empty',
        view=EmptyTrashCanView.as_view()
    ),
]

urlpatterns = []
urlpatterns.extend(urlpatterns_document_pages)
urlpatterns.extend(urlpatterns_document_types)
urlpatterns.extend(urlpatterns_document_versions)
urlpatterns.extend(urlpatterns_documents)
urlpatterns.extend(urlpatterns_duplicated_documents)
urlpatterns.extend(urlpatterns_favorite_documents)
urlpatterns.extend(urlpatterns_trashed_documents)

api_urls = [
    url(
        regex=r'^document_types/(?P<pk>[0-9]+)/$',
        view=APIDocumentTypeView.as_view(), name='documenttype-detail'
    ),
    url(
        regex=r'^document_types/$', view=APIDocumentTypeListView.as_view(),
        name='documenttype-list'
    ),
    url(
        regex=r'^document_types/(?P<pk>[0-9]+)/documents/$',
        view=APIDocumentTypeDocumentListView.as_view(),
        name='documenttype-document-list'
    ),
    url(
        regex=r'^documents/$', view=APIDocumentListView.as_view(),
        name='document-list'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/$', view=APIDocumentView.as_view(),
        name='document-detail'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/download/$',
        view=APIDocumentDownloadView.as_view(), name='document-download'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/type/change/$',
        view=APIDocumentDocumentTypeChangeView.as_view(),
        name='document-type-change'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/versions/$',
        view=APIDocumentVersionsListView.as_view(),
        name='document-version-list'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/versions/(?P<version_pk>[0-9]+)/$',
        view=APIDocumentVersionView.as_view(), name='documentversion-detail'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/versions/(?P<version_pk>[0-9]+)/pages/$',
        view=APIDocumentVersionPageListView.as_view(),
        name='documentversion-page-list'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/versions/(?P<version_pk>[0-9]+)/download/$',
        view=APIDocumentVersionDownloadView.as_view(),
        name='documentversion-download'
    ),
    url(
        regex=r'^documents/recent/$', view=APIRecentDocumentListView.as_view(),
        name='document-recent-list'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/versions/(?P<version_pk>[0-9]+)/pages/(?P<page_pk>[0-9]+)$',
        view=APIDocumentPageView.as_view(), name='documentpage-detail'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/versions/(?P<version_pk>[0-9]+)/pages/(?P<page_pk>[0-9]+)/image/$',
        view=APIDocumentPageImageView.as_view(), name='documentpage-image'
    ),
    url(
        regex=r'^trashed_documents/$',
        view=APITrashedDocumentListView.as_view(), name='trasheddocument-list'
    ),
    url(
        regex=r'^trashed_documents/(?P<pk>[0-9]+)/$',
        view=APIDeletedDocumentView.as_view(), name='trasheddocument-detail'
    ),
    url(
        regex=r'^trashed_documents/(?P<pk>[0-9]+)/restore/$',
        view=APIDeletedDocumentRestoreView.as_view(), name='trasheddocument-restore'
    ),
]



from unittest import skip

from selenium.common.exceptions import NoAlertPresentException

from django.conf import settings
from django.contrib.staticfiles.testing import StaticLiveServerTestCase

from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.common.tests.mixins import SeleniumTestMixin


@skip('Skip until a synchronous live server class is added.')
class BasePlainViewTestCase(
    SeleniumTestMixin, StaticLiveServerTestCase, GenericViewTestCase
):
    auto_add_test_view = True
    test_view_url = r'^javascript:alert\("XSS"\)/$'
    test_view_is_public = True
    test_view_template = 'javascript_view'

    def test_login_view_url_fragment_xss(self):
        # Should redirect and not display an alert
        self._open_url(
            fragment='#javascript:alert("XSS")', viewname=settings.LOGIN_URL
        )

        with self.assertRaises(expected_exception=NoAlertPresentException):
            self.webdriver.switch_to_alert()

    def test_login_view_url_redirect(self):
        self._open_url(
            fragment='#javascript:alert("XSS")', viewname=settings.LOGIN_URL
        )

        self.assertTrue(self.test_view_template in self.webdriver.page_source)



from ..models import Cabinet

from .literals import (
    TEST_CABINET_CHILD_LABEL, TEST_CABINET_LABEL, TEST_CABINET_LABEL_EDITED
)


class CabinetAPIViewTestMixin(object):
    def _request_test_cabinet_create_api_view(self, extra_data=None):
        data = {'label': TEST_CABINET_LABEL}

        if extra_data:
            data.update(extra_data)

        # Typecast to list to force queryset evaluation
        values = list(Cabinet.objects.values_list('pk', flat=True))

        response = self.post(viewname='rest_api:cabinet-list', data=data)

        self.test_cabinet = Cabinet.objects.exclude(pk__in=values).first()

        return response

    def _request_test_cabinet_delete_api_view(self):
        return self.delete(
            viewname='rest_api:cabinet-detail', kwargs={
                'pk': self.test_cabinet.pk
            }
        )

    def _request_test_cabinet_document_remove_api_view(self):
        return self.delete(
            viewname='rest_api:cabinet-document', kwargs={
                'pk': self.test_cabinet.pk, 'document_pk': self.test_document.pk
            }
        )

    def _request_test_cabinet_edit_api_patch_view(self):
        return self.patch(
            data={'label': TEST_CABINET_LABEL_EDITED}, kwargs={
                'pk': self.test_cabinet.pk
            }, viewname='rest_api:cabinet-detail'
        )

    def _request_test_cabinet_edit_api_put_view(self):
        return self.put(
            data={'label': TEST_CABINET_LABEL_EDITED}, kwargs={
                'pk': self.test_cabinet.pk
            }, viewname='rest_api:cabinet-detail'
        )

    def _request_test_cabinet_list_api_view(self):
        return self.get(viewname='rest_api:cabinet-list')


class CabinetTestMixin(object):
    def setUp(self):
        super(CabinetTestMixin, self).setUp()
        if not hasattr(self, 'test_cabinets'):
            self.test_cabinets = []

    def _create_test_cabinet(self):
        self.test_cabinet = Cabinet.objects.create(label=TEST_CABINET_LABEL)
        self.test_cabinets.append(self.test_cabinet)

    def _create_test_cabinet_child(self):
        self.test_cabinet_child = Cabinet.objects.create(
            label=TEST_CABINET_CHILD_LABEL, parent=self.test_cabinet
        )


class CabinetViewTestMixin(object):
    def setUp(self):
        super(CabinetViewTestMixin, self).setUp()
        if not hasattr(self, 'test_cabinets'):
            self.test_cabinets = []

    def _request_test_cabinet_create_view(self):
        # Typecast to list to force queryset evaluation
        values = list(Cabinet.objects.values_list('pk', flat=True))

        response = self.post(
            'cabinets:cabinet_create', data={
                'label': TEST_CABINET_LABEL
            }
        )

        self.test_cabinet = Cabinet.objects.exclude(pk__in=values).first()
        self.test_cabinets.append(self.test_cabinet)

        return response

    def _request_test_cabinet_delete_view(self):
        return self.post(
            viewname='cabinets:cabinet_delete', kwargs={
                'cabinet_id': self.test_cabinet.pk
            }
        )

    def _request_test_cabinet_edit_view(self):
        return self.post(
            viewname='cabinets:cabinet_edit', kwargs={
                'cabinet_id': self.test_cabinet.pk
            }, data={
                'label': TEST_CABINET_LABEL_EDITED
            }
        )

    def _request_test_cabinet_child_create_view(self):
        # Typecast to list to force queryset evaluation
        values = list(Cabinet.objects.values_list('pk', flat=True))

        response = self.post(
            viewname='cabinets:cabinet_child_add', kwargs={
                'cabinet_id': self.test_cabinet.pk
            }, data={'label': TEST_CABINET_CHILD_LABEL}
        )

        self.test_cabinet = Cabinet.objects.exclude(pk__in=values).first()
        self.test_cabinets.append(self.test_cabinet)

        return response

    def _request_test_cabinet_child_delete_view(self):
        return self.post(
            viewname='cabinets:cabinet_delete', kwargs={
                'cabinet_id': self.test_cabinet_child.pk
            }
        )

    def _request_test_cabinet_document_list_view(self):
        return self.get(
            viewname='cabinets:cabinet_view', kwargs={
                'cabinet_id': self.test_cabinet.pk
            }
        )

    def _request_test_cabinet_list_view(self):
        return self.get(viewname='cabinets:cabinet_list')

    def _request_test_document_cabinet_add_view(self):
        return self.post(
            viewname='cabinets:document_cabinet_add', kwargs={
                'document_id': self.test_document.pk
            }, data={
                'cabinets': self.test_cabinet.pk
            }
        )

    def _request_test_document_cabinet_multiple_remove_view(self):
        return self.post(
            viewname='cabinets:document_cabinet_remove', kwargs={
                'document_id': self.test_document.pk
            }, data={
                'cabinets': (self.test_cabinet.pk,),
            }
        )

    def _request_test_document_multiple_cabinet_multiple_add_view_cabinet(self):
        return self.post(
            viewname='cabinets:document_multiple_cabinet_add', data={
                'id_list': (self.test_document.pk,),
                'cabinets': self.test_cabinet.pk
            }
        )


class DocumentCabinetAPIViewTestMixin(object):
    def _request_test_document_cabinet_list_view(self):
        return self.get(
            viewname='rest_api:document-cabinet-list', kwargs={
                'pk': self.test_document.pk
            }
        )


class DocumentCabinetViewTestMixin(object):
    def _request_test_document_cabinet_list_view(self):
        return self.get(
            viewname='cabinets:document_cabinet_list', kwargs={
                'document_id': self.test_document.pk
            }
        )



from ..classes import WorkflowAction
from ..models import Workflow, WorkflowRuntimeProxy, WorkflowStateRuntimeProxy

from .literals import (
    TEST_WORKFLOW_INITIAL_STATE_LABEL, TEST_WORKFLOW_INITIAL_STATE_COMPLETION,
    TEST_WORKFLOW_INSTANCE_LOG_ENTRY_COMMENT, TEST_WORKFLOW_INTERNAL_NAME,
    TEST_WORKFLOW_LABEL, TEST_WORKFLOW_LABEL_EDITED,
    TEST_WORKFLOW_STATE_ACTION_LABEL, TEST_WORKFLOW_STATE_ACTION_LABEL_EDITED,
    TEST_WORKFLOW_STATE_ACTION_DOTTED_PATH, TEST_WORKFLOW_STATE_ACTION_WHEN,
    TEST_WORKFLOW_STATE_COMPLETION, TEST_WORKFLOW_STATE_LABEL,
    TEST_WORKFLOW_STATE_LABEL_EDITED,
    TEST_WORKFLOW_TRANSITION_FIELD_HELP_TEXT,
    TEST_WORKFLOW_TRANSITION_FIELD_LABEL,
    TEST_WORKFLOW_TRANSITION_FIELD_NAME, TEST_WORKFLOW_TRANSITION_FIELD_TYPE,
    TEST_WORKFLOW_TRANSITION_LABEL, TEST_WORKFLOW_TRANSITION_LABEL_EDITED,
    TEST_WORKFLOW_TRANSITION_LABEL_2
)


class TestWorkflowAction(WorkflowAction):
    label = 'test workflow state action'

    def execute(self, context):
        context['workflow_instance']._workflow_state_action_executed = True


class WorkflowRuntimeProxyStateViewTestMixin(object):
    def _request_test_workflow_runtime_proxy_state_list_view(self):
        return self.get(
            viewname='document_states:workflow_runtime_proxy_state_list',
            kwargs={'workflow_runtime_proxy_id': self.test_workflow.pk}
        )

    def _request_test_workflow_runtime_proxy_state_document_list_view(self):
        return self.get(
            viewname='document_states:workflow_runtime_proxy_state_document_list',
            kwargs={
                'workflow_runtime_proxy_state_id': self.test_workflow_state_1.pk
            }
        )


class WorkflowRuntimeProxyViewTestMixin(object):
    def _request_test_workflow_runtime_proxy_list_view(self):
        return self.get(
            viewname='document_states:workflow_runtime_proxy_list'
        )

    def _request_test_workflow_runtime_proxy_document_list_view(self):
        return self.get(
            viewname='document_states:workflow_runtime_proxy_document_list',
            kwargs={'workflow_runtime_proxy_id': self.test_workflow.pk}
        )


class WorkflowStateActionTestMixin(object):
    TestWorkflowAction = TestWorkflowAction
    test_workflow_state_action_path = 'mayan.apps.document_states.tests.mixins.TestWorkflowAction'

    def _create_test_workflow_state_action(self, workflow_state_index=0):
        self.test_workflow_state_action = self.test_workflow_states[
            workflow_state_index
        ].actions.create(
            label=self.TestWorkflowAction.label,
            action_path=self.test_workflow_state_action_path
        )


class WorkflowStateActionViewTestMixin(object):
    def _request_test_workflow_template_state_action_create_get_view(self, class_path):
        return self.get(
            viewname='document_states:workflow_template_state_action_create',
            kwargs={
                'workflow_template_state_id': self.test_workflow_state.pk,
                'class_path': class_path
            }
        )

    def _request_test_workflow_template_state_action_create_post_view(
        self, class_path, extra_data=None
    ):
        data = {
            'label': TEST_WORKFLOW_STATE_ACTION_LABEL,
            'when': TEST_WORKFLOW_STATE_ACTION_WHEN
        }
        if extra_data:
            data.update(extra_data)

        return self.post(
            viewname='document_states:workflow_template_state_action_create',
            kwargs={
                'workflow_template_state_id': self.test_workflow_state.pk,
                'class_path': class_path
            }, data=data
        )

    def _request_test_worflow_template_state_action_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_action_delete',
            kwargs={
                'workflow_template_state_action_id': self.test_workflow_state_action.pk
            }
        )

    def _request_test_worflow_template_state_action_edit_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_action_edit',
            kwargs={
                'workflow_template_state_action_id': self.test_workflow_state_action.pk
            }, data={
                'label': TEST_WORKFLOW_STATE_ACTION_LABEL_EDITED,
                'when': self.test_workflow_state_action.when
            }
        )

    def _request_test_worflow_template_state_action_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_state_action_list',
            kwargs={'workflow_template_state_id': self.test_workflow_state.pk}
        )

    def _request_test_workflow_state_action_selection_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_action_selection',
            kwargs={
                'workflow_template_state_id': self.test_workflow_state.pk
            }, data={'klass': TEST_WORKFLOW_STATE_ACTION_DOTTED_PATH}
        )


class WorkflowStateViewTestMixin(object):
    def _request_test_workflow_state_create_view(self, extra_data=None):
        data = {
            'label': TEST_WORKFLOW_STATE_LABEL,
            'completion': TEST_WORKFLOW_STATE_COMPLETION,
        }
        if extra_data:
            data.update(extra_data)

        return self.post(
            viewname='document_states:workflow_template_state_create',
            kwargs={'workflow_template_id': self.test_workflow.pk}, data=data
        )

    def _request_test_workflow_state_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_delete',
            kwargs={
                'workflow_template_state_id': self.test_workflow_state_1.pk
            }
        )

    def _request_test_workflow_state_edit_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_edit',
            kwargs={
                'workflow_template_state_id': self.test_workflow_state_1.pk
            }, data={
                'label': TEST_WORKFLOW_STATE_LABEL_EDITED
            }
        )

    def _request_test_workflow_state_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_state_list',
            kwargs={'workflow_template_id': self.test_workflow.pk}
        )


class WorkflowTestMixin(object):
    def _create_test_workflow(self, add_document_type=False):
        self.test_workflow = Workflow.objects.create(
            label=TEST_WORKFLOW_LABEL,
            internal_name=TEST_WORKFLOW_INTERNAL_NAME
        )
        self.test_workflow_runtime_proxy = WorkflowRuntimeProxy.objects.get(
            pk=self.test_workflow.pk
        )

        if add_document_type:
            self.test_workflow.document_types.add(self.test_document_type)

    def _create_test_workflow_state(self):
        self.test_workflow_states = []
        self.test_workflow_state = self.test_workflow.states.create(
            completion=TEST_WORKFLOW_STATE_COMPLETION, initial=True,
            label=TEST_WORKFLOW_STATE_LABEL
        )
        self.test_workflow_states.append(self.test_workflow_state)
        self.test_workflow_state_runtime_proxy = WorkflowStateRuntimeProxy.objects.get(
            pk=self.test_workflow_state.pk
        )

    def _create_test_workflow_states(self):
        self.test_workflow_states = []
        self.test_workflow_state_1 = self.test_workflow.states.create(
            completion=TEST_WORKFLOW_INITIAL_STATE_COMPLETION,
            initial=True, label=TEST_WORKFLOW_INITIAL_STATE_LABEL
        )
        self.test_workflow_states.append(self.test_workflow_state_1)
        self.test_workflow_state_2 = self.test_workflow.states.create(
            completion=TEST_WORKFLOW_STATE_COMPLETION,
            label=TEST_WORKFLOW_STATE_LABEL
        )
        self.test_workflow_states.append(self.test_workflow_state_2)
        self.test_workflow_state_runtime_proxy_1 = WorkflowStateRuntimeProxy.objects.get(
            pk=self.test_workflow_state_1.pk
        )
        self.test_workflow_state_runtime_proxy_2 = WorkflowStateRuntimeProxy.objects.get(
            pk=self.test_workflow_state_2.pk
        )

    def _create_test_workflow_transition(self):
        self.test_workflow_transition = self.test_workflow.transitions.create(
            label=TEST_WORKFLOW_TRANSITION_LABEL,
            origin_state=self.test_workflow_state_1,
            destination_state=self.test_workflow_state_2,
        )

    def _create_test_workflow_transitions(self):
        self.test_workflow_transition = self.test_workflow.transitions.create(
            workflow=self.test_workflow, label=TEST_WORKFLOW_TRANSITION_LABEL,
            origin_state=self.test_workflow_state_1,
            destination_state=self.test_workflow_state_2
        )

        self.test_workflow_transition_2 = self.test_workflow.transitions.create(
            workflow=self.test_workflow, label=TEST_WORKFLOW_TRANSITION_LABEL_2,
            origin_state=self.test_workflow_state_1,
            destination_state=self.test_workflow_state_2
        )

    def _create_test_workflow_instance_log_entry(self):
        self.test_document.workflows.first().log_entries.create(
            comment=TEST_WORKFLOW_INSTANCE_LOG_ENTRY_COMMENT,
            transition=self.test_workflow_transition,
            user=self._test_case_user
        )


class WorkflowToolViewTestMixin(object):
    def _request_workflow_launch_view(self):
        return self.post(
            viewname='document_states:tool_launch_workflows',
        )


class WorkflowTransitionEventViewTestMixin(object):
    def _request_test_workflow_transition_event_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_transition_events',
            kwargs={
                'workflow_template_transition_id': self.test_workflow_transition.pk
            }
        )


class WorkflowTransitionFieldViewTestMixin(object):
    def _request_test_workflow_transition_field_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_transition_field_list',
            kwargs={
                'workflow_template_transition_id': self.test_workflow_transition.pk
            }
        )

    def _request_workflow_transition_field_create_view(self):
        return self.post(
            viewname='document_states:workflow_template_transition_field_create',
            kwargs={
                'workflow_template_transition_id': self.test_workflow_transition.pk
            }, data={
                'field_type': TEST_WORKFLOW_TRANSITION_FIELD_TYPE,
                'name': TEST_WORKFLOW_TRANSITION_FIELD_NAME,
                'label': TEST_WORKFLOW_TRANSITION_FIELD_LABEL,
                'help_text': TEST_WORKFLOW_TRANSITION_FIELD_HELP_TEXT
            }
        )

    def _request_workflow_transition_field_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_transition_field_delete',
            kwargs={
                'workflow_template_transition_field_id': self.test_workflow_transition_field.pk
            },
        )


class WorkflowTransitionViewTestMixin(object):
    def _request_test_workflow_transition_create_view(self):
        return self.post(
            viewname='document_states:workflow_template_transition_create',
            kwargs={'workflow_template_id': self.test_workflow.pk}, data={
                'label': TEST_WORKFLOW_TRANSITION_LABEL,
                'origin_state': self.test_workflow_state_1.pk,
                'destination_state': self.test_workflow_state_2.pk,
            }
        )

    def _request_test_workflow_transition_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_transition_delete',
            kwargs={
                'workflow_template_transition_id': self.test_workflow_transition.pk
            }
        )

    def _request_test_workflow_transition_edit_view(self):
        return self.post(
            viewname='document_states:workflow_template_transition_edit',
            kwargs={
                'workflow_template_transition_id': self.test_workflow_transition.pk
            }, data={
                'label': TEST_WORKFLOW_TRANSITION_LABEL_EDITED,
                'origin_state': self.test_workflow_state_1.pk,
                'destination_state': self.test_workflow_state_2.pk,
            }
        )

    def _request_test_workflow_transition_execute_view(self):
        return self.post(
            viewname='document_states:workflow_instance_transition_execute',
            kwargs={
                'workflow_instance_id': self.test_workflow_instance.pk,
                'workflow_transition_id': self.test_workflow_transition.pk,
            }
        )

    def _request_test_workflow_transition_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_transition_list',
            kwargs={'workflow_template_id': self.test_workflow.pk}
        )

    def _request_test_workflow_transition_selection_get_view(self):
        return self.get(
            viewname='document_states:workflow_instance_transition_selection',
            kwargs={
                'workflow_instance_id': self.test_workflow_instance.pk,
            }
        )

    def _request_test_workflow_transition_selection_post_view(self):
        return self.post(
            viewname='document_states:workflow_instance_transition_selection',
            kwargs={
                'workflow_instance_id': self.test_workflow_instance.pk,
            }, data={
                'transition': self.test_workflow_transition.pk,
            }
        )


class WorkflowViewTestMixin(object):
    def _request_test_workflow_create_view(self):
        return self.post(
            viewname='document_states:workflow_template_create', data={
                'label': TEST_WORKFLOW_LABEL,
                'internal_name': TEST_WORKFLOW_INTERNAL_NAME,
            }
        )

    def _request_test_workflow_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_delete', kwargs={
                'workflow_template_id': self.test_workflow.pk
            }
        )

    def _request_test_workflow_edit_view(self):
        return self.post(
            viewname='document_states:workflow_template_edit', kwargs={
                'workflow_template_id': self.test_workflow.pk,
            }, data={
                'label': TEST_WORKFLOW_LABEL_EDITED,
                'internal_name': self.test_workflow.internal_name
            }
        )

    def _request_test_workflow_launch_view(self):
        return self.post(
            viewname='document_states:workflow_template_launch', kwargs={
                'workflow_template_id': self.test_workflow.pk
            }
        )

    def _request_test_workflow_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_list',
        )

    def _request_test_workflow_template_preview_view(self):
        return self.get(
            viewname='document_states:workflow_template_preview', kwargs={
                'workflow_template_id': self.test_workflow.pk,
            }
        )



from rest_framework import status

from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.permissions import permission_document_create
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..models.staging_folder_sources import StagingFolderSource
from ..permissions import (
    permission_sources_setup_create, permission_sources_setup_delete,
    permission_sources_setup_edit, permission_sources_setup_view,
    permission_staging_file_delete
)

from .mixins import (
    StagingFolderAPIViewTestMixin, StagingFolderFileAPIViewTestMixin,
    StagingFolderTestMixin
)


class StagingFolderAPIViewTestCase(
    StagingFolderAPIViewTestMixin, StagingFolderTestMixin, BaseAPITestCase
):
    def test_staging_folder_create_api_view_no_permission(self):
        staging_folder_count = StagingFolderSource.objects.count()

        response = self._request_test_staging_folder_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertEqual(
            StagingFolderSource.objects.count(), staging_folder_count
        )

    def test_staging_folder_create_api_view_with_permission(self):
        self.grant_permission(permission=permission_sources_setup_create)

        staging_folder_count = StagingFolderSource.objects.count()

        response = self._request_test_staging_folder_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertEqual(
            StagingFolderSource.objects.count(), staging_folder_count + 1
        )

    def test_staging_folder_delete_api_view_no_access(self):
        self._create_test_staging_folder()

        staging_folder_count = StagingFolderSource.objects.count()

        response = self._request_test_staging_folder_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            StagingFolderSource.objects.count(), staging_folder_count
        )

    def test_staging_folder_delete_api_view_with_permission(self):
        self._create_test_staging_folder()

        self.grant_permission(permission=permission_sources_setup_delete)

        staging_folder_count = StagingFolderSource.objects.count()

        response = self._request_test_staging_folder_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(
            StagingFolderSource.objects.count(), staging_folder_count - 1
        )

    def test_staging_folder_edit_api_view_via_patch_no_access(self):
        self._create_test_staging_folder()

        staging_folder_label = self.test_staging_folder.label

        response = self._request_test_staging_folder_edit_api_view(verb='patch')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_staging_folder.refresh_from_db()
        self.assertEqual(self.test_staging_folder.label, staging_folder_label)

    def test_staging_folder_edit_api_view_via_patch_with_permission(self):
        self._create_test_staging_folder()

        self.grant_permission(permission=permission_sources_setup_edit)

        staging_folder_label = self.test_staging_folder.label

        response = self._request_test_staging_folder_edit_api_view(verb='patch')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_staging_folder.refresh_from_db()
        self.assertNotEqual(
            self.test_staging_folder.label, staging_folder_label
        )

    def test_staging_folder_edit_api_view_via_put_no_access(self):
        self._create_test_staging_folder()

        staging_folder_label = self.test_staging_folder.label

        response = self._request_test_staging_folder_edit_api_view(
            extra_data={
                'folder_path': self.test_staging_folder.folder_path,
                'preview_width': self.test_staging_folder.preview_width,
                'uncompress': self.test_staging_folder.uncompress
            }, verb='put'
        )
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_staging_folder.refresh_from_db()
        self.assertEqual(self.test_staging_folder.label, staging_folder_label)

    def test_staging_folder_edit_api_view_via_put_with_permission(self):
        self._create_test_staging_folder()

        self.grant_permission(permission=permission_sources_setup_edit)

        staging_folder_label = self.test_staging_folder.label

        response = self._request_test_staging_folder_edit_api_view(
            extra_data={
                'folder_path': self.test_staging_folder.folder_path,
                'preview_width': self.test_staging_folder.preview_width,
                'uncompress': self.test_staging_folder.uncompress
            }, verb='put'
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_staging_folder.refresh_from_db()
        self.assertNotEqual(
            self.test_staging_folder.label, staging_folder_label
        )

    def test_staging_folder_api_list_api_view_no_permission(self):
        self._create_test_staging_folder()

        response = self._request_test_staging_folder_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staging_folder_api_list_api_view_with_permission(self):
        self._create_test_staging_folder()

        self.grant_permission(permission=permission_sources_setup_view)

        response = self._request_test_staging_folder_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['label'],
            self.test_staging_folder.label
        )


class StagingFolderFileAPIViewTestCase(
    DocumentTestMixin, StagingFolderFileAPIViewTestMixin,
    StagingFolderTestMixin, BaseAPITestCase
):
    auto_create_test_document_type = False
    auto_upload_test_document = False

    def test_staging_folder_file_delete_api_view_no_permission(self):
        self._create_test_staging_folder()
        self._copy_test_document()
        staging_file_count = len(list(self.test_staging_folder.get_files()))

        response = self._request_test_staging_folder_file_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertEqual(
            len(list(self.test_staging_folder.get_files())),
            staging_file_count
        )

    def test_staging_folder_file_delete_api_view_with_permission(self):
        self.grant_permission(permission=permission_staging_file_delete)
        self._create_test_staging_folder()
        self._copy_test_document()
        staging_file_count = len(list(self.test_staging_folder.get_files()))

        response = self._request_test_staging_folder_file_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        self.assertTrue(response.data is None)

        self.assertNotEqual(
            len(list(self.test_staging_folder.get_files())),
            staging_file_count
        )

    def test_staging_folder_file_detail_api_view(self):
        self._create_test_staging_folder()
        self._copy_test_document()

        response = self._request_test_staging_folder_file_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertTrue(
            response.data['image_url'].endswith(
                self.test_staging_folder_file.get_api_image_url()
            )
        )

    def test_staging_folder_file_upload_api_view_no_permission(self):
        self._create_test_document_type()
        self._create_test_staging_folder()
        self._copy_test_document()
        document_count = Document.objects.count()

        response = self._request_test_staging_folder_file_upload_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(Document.objects.count(), document_count)

    def test_staging_folder_file_upload_api_view_document_access(self):
        self._create_test_document_type()
        self._create_test_staging_folder()
        self._copy_test_document()
        document_count = Document.objects.count()

        self.grant_access(
            obj=self.test_document_type, permission=permission_document_create
        )

        response = self._request_test_staging_folder_file_upload_api_view()
        self.assertEqual(response.status_code, status.HTTP_202_ACCEPTED)
        self.assertEqual(Document.objects.count(), document_count + 1)



from django.contrib.contenttypes.models import ContentType
from django.shortcuts import get_object_or_404

from mayan.apps.permissions.serializers import PermissionSerializer
from mayan.apps.rest_api import generics

from .classes import ModelPermission
from .models import AccessControlList
from .permissions import permission_acl_edit, permission_acl_view
from .serializers import (
    AccessControlListPermissionSerializer, AccessControlListSerializer,
    WritableAccessControlListPermissionSerializer,
    WritableAccessControlListSerializer
)


class APIClassPermissionList(generics.ListAPIView):
    """
    Returns a list of all the available permissions for a class.
    """
    serializer_class = PermissionSerializer

    def get_content_type(self):
        return get_object_or_404(
            klass=ContentType, app_label=self.kwargs['app_label'],
            model=self.kwargs['model_name']
        )

    def get_queryset(self):
        return ModelPermission.get_for_class(
            klass=self.get_content_type().model_class()
        )


class APIObjectACLListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the object's access control lists
    post: Create a new access control list for the selected object.
    """
    def get_content_object(self):
        content_type = get_object_or_404(
            klass=ContentType, app_label=self.kwargs['app_label'],
            model=self.kwargs['model_name']
        )

        content_object = get_object_or_404(
            klass=content_type.model_class(), pk=self.kwargs['object_id']
        )

        if self.request.method == 'GET':
            permission_required = permission_acl_view
        else:
            permission_required = permission_acl_edit

        AccessControlList.objects.check_access(
            obj=content_object, permissions=(permission_required,),
            user=self.request.user
        )

        return content_object

    def get_queryset(self):
        return self.get_content_object().acls.all()

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(APIObjectACLListView, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'content_object': self.get_content_object(),
                }
            )

        return context

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIObjectACLListView, self).get_serializer(
            *args, **kwargs
        )

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return AccessControlListSerializer
        else:
            return WritableAccessControlListSerializer


class APIObjectACLView(generics.RetrieveDestroyAPIView):
    """
    delete: Delete the selected access control list.
    get: Returns the details of the selected access control list.
    """
    serializer_class = AccessControlListSerializer

    def get_content_object(self):
        if self.request.method == 'GET':
            permission_required = permission_acl_view
        else:
            permission_required = permission_acl_edit

        content_type = get_object_or_404(
            klass=ContentType, app_label=self.kwargs['app_label'],
            model=self.kwargs['model_name']
        )

        content_object = get_object_or_404(
            klass=content_type.model_class(), pk=self.kwargs['object_id']
        )

        AccessControlList.objects.check_access(
            obj=content_object, permissions=(permission_required,),
            user=self.request.user
        )

        return content_object

    def get_queryset(self):
        return self.get_content_object().acls.all()


class APIObjectACLPermissionListView(generics.ListCreateAPIView):
    """
    get: Returns the access control list permission list.
    post: Add a new permission to the selected access control list.
    """
    def get_acl(self):
        return get_object_or_404(
            klass=self.get_content_object().acls, pk=self.kwargs['pk']
        )

    def get_content_object(self):
        content_type = get_object_or_404(
            klass=ContentType, app_label=self.kwargs['app_label'],
            model=self.kwargs['model_name']
        )

        content_object = get_object_or_404(
            klass=content_type.model_class(), pk=self.kwargs['object_id']
        )

        if self.request.method == 'GET':
            permission = permission_acl_view
        else:
            permission = permission_acl_edit

        AccessControlList.objects.check_access(
            obj=content_object, permissions=(permission,),
            user=self.request.user
        )

        return content_object

    def get_queryset(self):
        return self.get_acl().permissions.all()

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIObjectACLPermissionListView, self).get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return AccessControlListPermissionSerializer
        else:
            return WritableAccessControlListPermissionSerializer

    def get_serializer_context(self):
        context = super(
            APIObjectACLPermissionListView, self
        ).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'acl': self.get_acl(),
                }
            )

        return context


class APIObjectACLPermissionView(generics.RetrieveDestroyAPIView):
    """
    delete: Remove the permission from the selected access control list.
    get: Returns the details of the selected access control list permission.
    """
    lookup_url_kwarg = 'permission_pk'
    serializer_class = AccessControlListPermissionSerializer

    def get_acl(self):
        return get_object_or_404(
            klass=self.get_content_object().acls, pk=self.kwargs['pk']
        )

    def get_content_object(self):
        content_type = get_object_or_404(
            klass=ContentType, app_label=self.kwargs['app_label'],
            model=self.kwargs['model_name']
        )

        content_object = get_object_or_404(
            klass=content_type.model_class(), pk=self.kwargs['object_id']
        )

        if self.request.method == 'GET':
            permission = permission_acl_view
        else:
            permission = permission_acl_edit

        AccessControlList.objects.check_access(
            obj=content_object, permissions=(permission,),
            user=self.request.user
        )

        return content_object

    def get_queryset(self):
        return self.get_acl().permissions.all()

    def get_serializer_context(self):
        context = super(
            APIObjectACLPermissionView, self
        ).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'acl': self.get_acl(),
                }
            )

        return context
